#!/usr/bin/env python
import sys
import argparse
import traceback
import logging

import envi.common as e_common

import vdb
import vtrace
import vtrace.snapshot as vt_snap


logger = logging.getLogger('vdb')


def targetusage():
    print('''
    Alternate vdb "targets" include various embedded/hardware/etc debugging
    capabilities which are not related to your runtime platform....  Some
    targets may require additional foo=bar style options added to the end
    of your vdb command.  See details per-target below.

    ======================================================================

    "vmware32" host=<host> port=<port>
        ( of listening vmware gdb server for 32 bit VM )

        The "vmware32" target enables debugging with a 32bit VMWare Hypervisor
        using their gdb-server stub.  This allows "hardware debugger" like
        kernel debugging.

        Add the following lines to your VMX file to enable debugging:

        debugStub.listen.guest32 = "TRUE"
        debugStub.hideBreakpoints = "FALSE"

        VMWare will be listening on 127.0.0.1 on port 8832 (for the first one...
        8833 for the second... etc... )

        NOTE: Only Windows XP currently supported.
        TODO: Windows 7 / Linux / Windows 8

    ======================================================================

    In development:

    "vmware64"  - Kernel debug 64 bit VMWare Hypervisors
    "openocd"   - Debug embedded targets

    ''')
    sys.exit(0)

loglevels = (logging.CRITICAL,
             logging.ERROR,
             logging.WARN,
             logging.INFO,
             logging.DEBUG)


def main():
    parser = argparse.ArgumentParser(prog='vdbbin', usage='%(prog)s [options] [platformopt=foo, ...]')
    parser.add_argument('-c', '--cmd', dest='command', default=None, help='Debug a fired command')
    parser.add_argument('-p', '--process', dest='process', default=None, help='Attach to process by name or pid')
    parser.add_argument('-Q', '--qt', dest='doqt', default=False, action='store_true', help='Run the QT gui')
    parser.add_argument('-R', '--remote', dest='remotehost', default=None, help='Attach to remote VDB server')
    parser.add_argument('-r', '--run', dest='dorunagain', default=False, action='store_true', help='Do not stop on attach')
    parser.add_argument('-s', '--snapshot', dest='snapshot', default=None, help='Load a vtrace snapshot file')
    parser.add_argument('-S', '--server', dest='doserver', default=False, action='store_true')
    parser.add_argument('-v', '--verbose', dest='verbose', default=False, action='count')
    parser.add_argument('-t', '--target', dest='target', default=None, help='Activate special vdb target ( -t ? for list )')
    parser.add_argument('--android', dest='doandroid', default=False, action='store_true', help='Debug Android with ADB!')
    parser.add_argument('-e', '--eventid', dest='eventid', default=None, type=int, help='Used for Windows JIT')
    parser.add_argument('-w', '--waitfor', dest='waitfor', default=None, help='Wait for process with name')
    parser.add_argument('--LI', dest='BpLibInit', default=False, action='store_true', help='Breakpoint on Library initialization')
    parser.add_argument('--LL', dest='BpLibLoad', default=False, action='store_true', help='Breakpoint on Library load time')
    parser.add_argument('platargs', nargs='*')

    args = parser.parse_args()

    # setup logging
    verbose = min(args.verbose, 5)
    level = e_common.LOG_LEVELS[verbose]
    e_common.initLogging(logger, level=level)

    # Handle some options before we even create a trace.
    vtrace.remote = args.remotehost  # None by default

    platargs = {}

    for arg in args.platargs:

        if arg.find('=') == -1:
            continue

        k, v = arg.split('=', 1)
        if v.isdigit():
            v = int(v)

        platargs[k.lower()] = v

    if args.doandroid:
        import vdb.tools.android as v_utils_android
        v_utils_android.runVdbOnDroid()
        return

    if args.target == '?':
        targetusage()

    trace = None
    if args.snapshot:
        logger.info('Loading process snapshot...')
        trace = vt_snap.loadSnapshot(args.snapshot)

    if trace is None:
        trace = vtrace.getTrace(target=args.target, **platargs)

    db = vdb.Vdb(trace)
    db.runagain = args.dorunagain
    db.windows_jit_event = args.eventid

    # store setting in the Vdb Config for future traces
    db.config.vdb.BreakOnLibraryLoad = args.BpLibLoad
    db.config.vdb.BreakOnLibraryInit = args.BpLibInit
    # set the trace appropriately
    db.trace.setBreakOnLibraryLoad(args.BpLibLoad)
    db.trace.setBreakOnLibraryInit(args.BpLibInit)

    if args.waitfor:
        while True:
            newest_pid = 0
            for pid, pname in trace.ps():
                pname = pname.split(' ')[0]
                if pname.find(args.waitfor) != -1:
                    newest_pid = pid

            if newest_pid != 0:
                trace.attach(newest_pid)
                break

    if args.doqt:
        import vqt.main as vq_main
        import vdb.qt.main as vdb_q_main
        import vqt.colors as vq_colors
        vq_main.startup(css=vq_colors.qt_matrix)
        qgui = vdb_q_main.VdbWindow(db)
        qgui.show()

    if args.doserver:
        db.do_server('')

    if args.process:
        db.do_attach(args.process)

    elif args.command:
        trace.execute(args.command)

    if args.eventid:
        db.trace.setMeta('WindowsJitEvent', args.eventid)

    if args.doqt:
        vq_main.main()

    else:

        while not db.shutdown.isSet():
            try:
                db.cmdloop()
            except KeyboardInterrupt:
                if db.trace.isRunning():
                    db.trace.sendBreak()
            except SystemExit:
                break
            except:
                logger.error(traceback.format_exc())


if __name__ == '__main__':
    main()




'''
Render a graph full of gtk widgets and draw some lines!
'''

import gtk
import gtk.gdk as gdk

import vwidget.main as vw_main

import visgraph.renderers as vg_render
import visgraph.drawing.bezier as vw_bezier
import visgraph.drawing.catmullrom as vw_catmullrom

zero_zero = (0,0)

class GtkVisGraphOverview(gtk.DrawingArea):

    def __init__(self, graph, layout, scrollwin=None, ):
        gtk.DrawingArea.__init__(self)
        self._vg_graph = graph
        self._vg_layout = layout
        self._vg_scrollwin = scrollwin

        self.connect('expose-event', self.expose_event_cb)
        self.connect('button_press_event', self.button_press_event)
        self.set_events( self.get_events() | gtk.gdk.BUTTON_PRESS_MASK)

        self.modify_bg(gtk.STATE_NORMAL, gtk.gdk.Color())
        self.modify_fg(gtk.STATE_NORMAL, gtk.gdk.Color(green=65535))

        if scrollwin is not None:
            scrollwin.connect('destroy', self.scroll_destroy_cb)

    def scroll_destroy_cb(self, widget):
        self.destroy()

    def expose_event_cb(self, layout, event):

        style = self.get_style()
        gc = style.fg_gc[gtk.STATE_NORMAL]

        rect = self.get_allocation()

        owidth = rect.width
        oheight = rect.height

        lwidth, lheight = self._vg_layout.getLayoutSize()

        for nid, ninfo in self._vg_graph.getNodes():

            nwidth, nheight = ninfo.get('size', zero_zero)
            if nwidth == 0:
                continue

            xpos, ypos = ninfo.get('position', zero_zero)

            drawx = ((xpos * owidth) / lwidth)
            drawy = (ypos * oheight) / lheight

            sizex = owidth * nwidth / lwidth
            sizey = oheight * nheight / lheight

            colorstr = ninfo.get('color')
            if colorstr is None:
                colorstr = '#0f0'

            color = gtk.gdk.color_parse(colorstr)

            #self.modify_fg(gtk.STATE_NORMAL, color)
            self.window.draw_rectangle(gc, False, drawx, drawy, sizex, sizey)

            #c = self.window.cairo_create()
            #c.set_source_rgb(color.red / float(65535), color.green / float(65535), color.blue / float(65535))
            #c.rectangle(drawx, drawy, sizex, sizey)# event.area.x, event.area.y, event.area.width, event.area.height)
            #c.set_line_width(0.5)
            #c.stroke()

    def button_press_event(self, widget, event):
        if self._vg_scrollwin:
            rect = self.get_allocation()
            xper = event.x / float(rect.width)
            yper = event.y / float(rect.height)

            hadj = self._vg_scrollwin.get_hadjustment()
            vadj = self._vg_scrollwin.get_vadjustment()

            hvis = hadj.page_size
            vvis = vadj.page_size

            hadj.value = min(max((hadj.upper * xper) - (hvis / 2), 0), hadj.upper - hvis)
            vadj.value = min(max((vadj.upper * yper) - (vvis / 2), 0), vadj.upper - vvis)

            self._vg_scrollwin.set_hadjustment(hadj)
            self._vg_scrollwin.set_vadjustment(vadj)

#FIXME use cairo to draw!
class GtkVisGraphRenderer(gtk.Layout, vg_render.GraphRenderer):

    def __init__(self, graph):
        gtk.Layout.__init__(self)
        vg_render.GraphRenderer.__init__(self, graph)

        self._vg_lines = []

        self.connect('expose-event', self.expose_event_cb)
        self.connect('button_press_event', self.button_press_event)

        self.set_events( self.get_events() | gtk.gdk.BUTTON_PRESS_MASK)

        self.modify_bg(gtk.STATE_NORMAL, gtk.gdk.Color())
        self.modify_fg(gtk.STATE_NORMAL, gtk.gdk.Color(green=65535))

    def beginRender(self, width, height):
        vg_render.GraphRenderer.beginRender(self, width, height)
        self._vg_lines = []
        self.set_size(width, height)

    def renderNode(self, nid, ninfo, xpos, ypos):
        widget = ninfo.get('widget')
        # FIXME honor color, etc...?
        if widget is not None:
            self.move(widget, xpos, ypos)

    def renderEdge(self, eid, einfo, points):
        # FIXME deal with colors etc...
        self._vg_lines.append(points)

    def setNodeSizes(self, graph):

        for nid, ninfo in graph.getNodes():

            # Skip "ghost" nodes...
            if ninfo.get('ghost'):
                continue

            widget = ninfo.get('widget')
            if widget is None:
                widget = gtk.Label(str(nid))
                widget.modify_bg(gtk.STATE_NORMAL, gtk.gdk.Color())
                widget.modify_fg(gtk.STATE_NORMAL, gtk.gdk.Color(green=65535))
                ninfo['widget'] = widget

            # Put them all at 0,0 for now...
            self.put(widget, 0, 0)

        # Get them all to render...
        self.show_all()
        vw_main.doiterations()

        # Now that we have rendered them...
        for nid, ninfo in graph.getNodes():
            widget = ninfo.get('widget')
            if widget is None:
                continue

            size = widget.size_request()
            ninfo['size'] = size

    def button_press_event(self, widget, event):
        pass

    def expose_event_cb(self, layout, event):

        style = self.get_style()

        gc = style.fg_gc[gtk.STATE_NORMAL]

        for points in self._vg_lines:
            self.bin_window.draw_lines(gc, points)



from PyQt5.QtWidgets import QVBoxLayout

import vtrace.qt
import vdb.qt.base

from vqt.main import *

class VdbRegistersWindow(vdb.qt.base.VdbWidgetWindow):
    def __init__(self, db, dbt, parent=None):
        vdb.qt.base.VdbWidgetWindow.__init__(self, db, dbt, parent=parent)

        self.regsWidget = vtrace.qt.RegistersView(trace=dbt, parent=parent)

        vbox = QVBoxLayout()
        vbox.addWidget(self.regsWidget)
        self.setLayout(vbox)

        self.setWindowTitle('Registers')

        vqtconnect(self.vqLoad, 'vdb:setregs')
        vqtconnect(self.vqLoad, 'vdb:setthread')

    def vqLoad(self):
        '''
        the widgets in RegistersView already register for notifications.
        '''
        self.regsWidget.reglist.vqLoad()



import unittest

from vivisect.tests.helpers import MockVw

from vivisect.symboliks.common import *
from vivisect.symboliks.expression import symexp
from vivisect.symboliks.tests.helpers import MockEmulator


class TestConstraints(unittest.TestCase):
    '''
    Tests that symbolik constraints work the way we think
    '''
    def assertTruth(self, s1):
        v1 = symexp(s1).solve()
        self.assertTrue(v1 == 1)

    def assertReduce(self, s1, s2):
        sym1 = symexp(s1).reduce()
        sym2 = symexp(s2)
        self.assertEqual(str(sym1), str(sym2))

    @unittest.skip('This takes *forever* to compute')
    def test_perf(self):
        v1 = symexp('biz = bar ** foo + baz')
        v1.reduce()

    def test_const(self):
        self.assertTruth('0 < 1')
        self.assertTruth('0xab == 0xab')
        self.assertTruth('0xf00 > 0xabc')
        self.assertTruth('10 >= 9')
        self.assertTruth('9 <= 9')
        self.assertTruth('4 != 5')

    def test_op(self):
        self.assertTruth('foo == foo')

    def test_reduce(self):
        s1 = '((0 & ((foo * 4) >> 2)) == 0)'
        v1 = symexp('((0 & ((foo * 4) >> 2)) == 0)')
        self.assertTrue(s1 == str(v1))
        self.assertReduce(s1, '1')
        self.assertTrue(v1.getWidth() == 4)  # default width

        s2 = '((((foo * bar) * baz) + (131 | 40)) == (((foo * bar) * baz) + 171))'
        v2 = symexp(s2)
        self.assertTrue(s2 == str(v2))
        self.assertTrue(v2.reduce() == 1)
        self.assertTrue(v2.getWidth() == 4)  # default width

        s3 = '(((((foo - bar) + bar) + (131 | 40)) - 171) == foo)'
        v3 = symexp(s3)
        self.assertTrue(s3 == str(v3))
        self.assertTrue(v3.reduce() == 1)
        self.assertTrue(v3.getWidth() == 4)  # default width

        s4 = 'foo - (foo + 3 ** 4) == biz / bar + boo'
        v4 = symexp(s4)
        self.assertTrue(v4.reduce() != 0)

    def test_rev(self):
        v1 = symexp('4 < 5')
        v2 = symexp('4 >= 5')
        self.assertTrue(v1.isDiscrete())
        self.assertTrue(v2.isDiscrete())
        self.assertTrue(str(v1.reverse()) == str(v2))

        v1 = symexp('foo > bar')
        v2 = symexp('foo <= bar')
        self.assertFalse(v1.isDiscrete())
        self.assertFalse(v2.isDiscrete())
        self.assertTrue(str(v1.reverse()) == str(v2))

        v1 = symexp('foo == bar')
        v2 = symexp('foo > bar')
        self.assertFalse(v1.isDiscrete())
        self.assertFalse(v2.isDiscrete())
        self.assertFalse(str(v1.reverse()) == str(v2))

    def test_eq(self):
        v1 = symexp('10 != 5')
        v2 = symexp('2 * 5 != 3 + 2')
        self.assertTrue(v1 == v2)

        v1 = symexp('foo < bar')
        v2 = symexp('bar >= foo')
        self.assertTrue(v1 == v2)

    def test_walktree(self):
        v1 = symexp('((foo ** 2) << 1) + (bar * 3) - 4 > 0')

        def walker(path, kid, ctx):
            order.append(kid)
        order = []
        v1.walkTree(walker, ctx=order)
        correct = ['foo',
                   '2',
                   '(foo ** 2)',
                   '1',
                   '((foo ** 2) << 1)',
                   'bar',
                   '3',
                   '(bar * 3)',
                   '(((foo ** 2) << 1) + (bar * 3))',
                   '4',
                   '((((foo ** 2) << 1) + (bar * 3)) - 4)',
                   '0',
                   '(((((foo ** 2) << 1) + (bar * 3)) - 4) > 0)']
        if len(order) != len(correct):
            self.fail('test_walktree, visit produced unexpected results')

        for i in range(len(order)):
            self.assertTrue(str(order[i]) == correct[i])

    def test_update(self):
        v1 = symexp('foo * bar > bar << baz')
        self.assertFalse(v1.isDiscrete())
        emu = MockEmulator(MockVw())
        emu.setSymVariable('foo', Const(5, emu.__width__))
        emu.setSymVariable('bar', Const(2, emu.__width__))
        emu.setSymVariable('baz', Const(6, emu.__width__))
        v1 = v1.update(emu)
        self.assertTrue(v1.isDiscrete(emu))
        self.assertTrue(v1.solve(emu) == 0)

    def test_mixed(self):
        v1 = symexp('(foo ** 2) << (1 > 0)')
        emu = MockEmulator(MockVw())
        emu.setSymVariable('foo', Const(5, emu.__width__))
        v1 = v1.update(emu).reduce()
        self.assertTrue(v1 == 50)

    def test_layered(self):
        v1 = symexp('(1595 == 1595) == (47 == 2)')
        self.assertTrue(v1.reduce() == 0)

        v1 = symexp('(foo == foo) == (bar == bar)')
        self.assertTrue(v1.reduce() == 1)

        v2 = symexp('(foo == foo) & (bar == bar)')
        self.assertTrue(v2.reduce() == 1)
        self.assertTrue(v1.reduce() == v2.reduce())




"""
An analysis module that looks for likely *code* pointers by
checking for them to be pointing to imports and having
"call [deref]" bytes before them.
"""

from vivisect.const import *


def analyze(vw):

    for va, dest in vw.findPointers():
        # Is there a location already at the target?
        loc = vw.getLocation(dest)
        if loc is None:
            continue

        if loc[L_LTYPE] != LOC_IMPORT:
            continue

        offset, bytes = vw.getByteDef(va)
        if offset < 2:
            continue

        if bytes[offset-2:offset] == b"\xff\x15":  # call [importloc]
            # If there's a pointer here, remove it.
            if vw.getLocation(va):
                vw.delLocation(va)
            vw.makeCode(va-2)



import envi.archs.i386 as e_i386
import envi.archs.amd64 as e_amd64

import vivisect.impemu.emulator as v_i_emulator
import vivisect.impemu.platarch.i386 as v_i_i386
import vivisect.impemu.platarch.amd64 as v_i_amd64

from vivisect.impemu.emulator import imphook

MAX_PATH = 260

# A shared place for most of the import hooks
class WindowsMixin(v_i_emulator.WorkspaceEmulator):

    def __init__(self):
        pass

    def readFilePath(self, va, unicode=False, default='unknown'):
        size = MAX_PATH
        if unicode:
            size = MAX_PATH * 2
        bytez = self.readMemory(va, size)
        if unicode:
            bytez = bytez.decode('utf-16le', 'ignore')
            bytez = bytez.split('\x00')[0]
        else:
            bytez = bytez.split(b'\x00')[0]
            bytez = bytez.decode('utf-8', 'ignore')

        if len(bytez) == MAX_PATH:
            bytez = default

        return bytez

    def readLibraryPath(self, va, unicode=False):
        bytez = self.readFilePath(va, unicode=unicode, default='unknownlib')
        return bytez.lower().split('.')[0]

    @imphook('kernel32.LoadLibraryA')
    def kernel32_LoadLibraryA(self, emu, callconv, api, argv):
        lpLibName = argv[0]
        libname = self.readLibraryPath(lpLibName)
        retval = emu.setVivTaint('dynlib',libname)
        callconv.execCallReturn(emu, retval, len(argv))

    @imphook('kernel32.LoadLibraryW')
    def kernel32_LoadLibraryW(self, emu, callconv, api, argv):
        lpLibName = argv[0]
        libname = self.readLibraryPath(lpLibName, unicode=True)
        retval = emu.setVivTaint('dynlib',libname)
        callconv.execCallReturn(emu, retval, len(argv))

    @imphook('kernel32.GetProcAddress')
    def kernel32_GetProcAddress(self, emu, callconv, api, argv):
        hLibrary,lpProcName = argv
        procname = self.readFilePath(lpProcName, default='unknownproc')

        libname = 'unknownlib'
        taint = emu.getVivTaint(hLibrary)
        if taint:
            tva,ttype,tinfo = taint
            if ttype == 'dynlib':
                libname = tinfo

        retval = emu.setVivTaint('dynfunc', (libname, procname))
        callconv.execCallReturn(emu, retval, len(argv))

    @imphook('kernel32.GetModuleHandleA')
    def kernel32_GetModuleHandleA(self, emu, callconv, api, argv):
        return self.kernel32_LoadLibraryA(emu, callconv, api, argv)

    @imphook('kernel32.GetModuleHandleW')
    def kernel32_GetModuleHandleW(self, emu, callconv, api, argv):
        return self.kernel32_LoadLibraryW(emu, callconv, api, argv)

    @imphook('kernel32.LoadLibraryExA')
    def kernel32_LoadLibraryExA(self, emu, callconv, api, argv):
        return self.kernel32_LoadLibraryA(emu, callconv, api, argv)

    @imphook('kernel32.LoadLibraryExW')
    def kernel32_LoadLibraryExW(self, emu, callconv, api, argv):
        return self.kernel32_LoadLibraryW(emu, callconv, api, argv)

    @imphook('kernel32.GetModuleHandleExA')
    def kernel32_GetModuleHandleExA(self, emu, callconv, api, argv):
        dwFlags,lpLibName,phModule = argv
        libname = self.readLibraryPath(lpLibName, unicode=False)
        retval = emu.setVivTaint('dynlib',libname)
        callconv.execCallReturn(emu, retval, len(argv))

    @imphook('kernel32.GetModuleHandleExW')
    def kernel32_GetModuleHandleExW(self, emu, callconv, api, argv):
        dwFlags,lpLibName,phModule = argv
        libname = self.readLibraryPath(lpLibName, unicode=True)
        retval = emu.setVivTaint('dynlib',libname)
        callconv.execCallReturn(emu, retval, len(argv))

class Windowsi386Emulator(WindowsMixin, v_i_i386.i386WorkspaceEmulator):

    taintregs = [
        e_i386.REG_EAX, e_i386.REG_ECX, e_i386.REG_EDX,
        e_i386.REG_EBX, e_i386.REG_EBP, e_i386.REG_ESI,
        e_i386.REG_EDI,
    ]

    def __init__(self, vw, **kwargs):
        '''
        Please see the base emulator class in vivisect/impemu/emulator.py for the parameters
        that can be passed through kwargs
        '''
        v_i_i386.i386WorkspaceEmulator.__init__(self, vw, **kwargs)
        WindowsMixin.__init__(self)

    @imphook('ntdll.seh3_prolog')
    def seh3_prolog(self, emu, callconv, api, argv):

        scopetable, localsize = argv

        emu.doPush(0) # seh3_handler
        emu.doPush(0) # saved seh3 scopetable

        ebp = emu.getRegister(e_i386.REG_EBP)
        esp = emu.getRegister(e_i386.REG_ESP)
        emu.writeMemValue(esp+16, ebp, 4)

        ebp = esp+16 # [saved_scope, seh3_handler, saved_eip, new_scope, <size>]
        esp -= localsize

        emu.setRegister(e_i386.REG_EBP, ebp)
        emu.setRegister(e_i386.REG_ESP, esp)
        emu.doPush(emu.getRegister(e_i386.REG_EBX))
        emu.doPush(emu.getRegister(e_i386.REG_ESI))
        emu.doPush(emu.getRegister(e_i386.REG_EDI))

    @imphook('ntdll.seh4_prolog')
    def seh4_prolog(self, emu, callconv, api, argv):
        self.seh3_prolog(emu, callconv, api, argv)
        emu.doPush(0xc0c0c0c0)
        emu.doPush(0)

    @imphook('ntdll.seh4_gs_prolog')
    def seh4_gs_prolog(self, emu, callconv, api, argv):
        self.seh3_prolog(emu, callconv, api, argv)
        emu.doPush(0xc0c0c0c0)
        emu.doPush(0)

    @imphook('ntdll.seh3_epilog')
    def seh3_epilog(self, emu, callconv, api, argv):

        emu.doPop()
        emu.setRegister(e_i386.REG_EDI, emu.doPop())
        emu.setRegister(e_i386.REG_ESI, emu.doPop())
        emu.setRegister(e_i386.REG_EBX, emu.doPop())

        ebp = emu.getRegister(e_i386.REG_EBP)
        emu.setRegister(e_i386.REG_ESP, ebp)
        emu.setRegister(e_i386.REG_EBP, emu.doPop())

    @imphook('ntdll.seh4_epilog')
    def seh4_epilog(self, emu, callconv, api, argv):

        emu.doPop()
        emu.setRegister(e_i386.REG_EDI, emu.doPop())
        emu.setRegister(e_i386.REG_EDI, emu.doPop())
        emu.setRegister(e_i386.REG_ESI, emu.doPop())
        emu.setRegister(e_i386.REG_EBX, emu.doPop())

        ebp = emu.getRegister(e_i386.REG_EBP)
        emu.setRegister(e_i386.REG_ESP, ebp)
        emu.setRegister(e_i386.REG_EBP, emu.doPop())

    @imphook('ntdll.eh_prolog')
    def eh_prolog(self, emu, callconv, api, argv):
        emu.doPop()  # Remove saved eip

        # Push ebp, move ebp, esp
        emu.doPush(emu.getRegister(e_i386.REG_EBP))
        esp = emu.getRegister(e_i386.REG_ESP)
        emu.setRegister(e_i386.REG_EBP, esp)

        # Push a new EH record
        emu.doPush(0xffffffff)
        emu.doPush(emu.getRegister(e_i386.REG_EAX))
        emu.doPush(0xc0c0c0c0)

    @imphook('ntdll._alloca_probe')
    def alloca_probe(self, emu, callconv, api, argv):
        '''
        Causes ESP to be decremented by the value passed in EAX.
        The actual function loops over each 4k page (0x1000 bytes) and
        forces a memory access, presumably to trigger any O/S page faults.
        The emulation here does not loop or read memory.

        Results:  ESP decremented by EAX
                  EAX set to EIP of next instruction

        Safeguard ESP to be nonnegative.  The true alloca_probe code has this
        safeguard, though in practice an illegal memory access O/S exception
        is raised.
        '''
        esp = emu.getRegister(e_i386.REG_ESP)
        eax = emu.getRegister(e_i386.REG_EAX)
        eip = emu.getRegister(e_i386.REG_EIP)

        new_esp = max(esp + 4 - eax, 0)
        emu.setRegister(e_i386.REG_ESP, new_esp)
        ret_addr = self.readMemValue(esp, size=4)
        emu.setRegister(e_i386.REG_EAX, ret_addr)

    @imphook('ntdll.gs_prolog')
    def gs_prolog(self, emu, callconv, api, argv):
        esp = emu.getRegister(e_i386.REG_ESP)
        esp += 8
        eax = emu.getRegister(e_i386.REG_EAX)
        # XXX - this is not a complete implementation..
        if eax < 0x1000:
            emu.setRegister(e_i386.REG_ESP,  (esp-eax))

class WindowsAmd64Emulator(WindowsMixin, v_i_amd64.Amd64WorkspaceEmulator):

    taintregs = [
        e_amd64.REG_RAX, e_amd64.REG_RCX, e_amd64.REG_RDX,
        e_amd64.REG_RBX, e_amd64.REG_RBP, e_amd64.REG_RSI,
        e_amd64.REG_RDI, e_amd64.REG_R8,  e_amd64.REG_R9,
    ]

    def __init__(self, vw, **kwargs):
        '''
        Please see the base emulator class in vivisect/impemu/emulator.py for the parameters
        that can be passed through kwargs
        '''
        v_i_amd64.Amd64WorkspaceEmulator.__init__(self, vw, **kwargs)
        WindowsMixin.__init__(self)

    @imphook('ntdll._alloca_probe')
    def alloca_probe(self, emu, callconv, api, argv):
        '''
        Return RSP unchanged.  This requires a correction to the stack since
        the emulator will push a return value.  The actual alloca_probe has a
        nonstandard calling convention.

        The actual function loops over each 4k page (0x1000 bytes) and
        forces a memory access, presumably to trigger any O/S page faults.
        The emulation here does not loop or read memory.
        '''
        emu.doPop()



# calling convention constants
CC_REG              = 1 << 0    # argument is stored in a register
CC_STACK            = 1 << 1    # argument is stored on the stack
CC_STACK_INF        = 1 << 2    # all following args are stored on the stack
CC_CALLEE_CLEANUP   = 1 << 3    # callee cleans up the stack
CC_CALLER_CLEANUP   = 1 << 4    # caller cleans up the stack

# meta-register constants
RMETA_MASK  = 0xffff0000
RMETA_NMASK = 0x0000ffff

ENDIAN_LSB = 0
ENDIAN_MSB = 1

# Memory Map Permission Flags
MM_NONE = 0x0
MM_READ = 0x4
MM_WRITE = 0x2
MM_EXEC = 0x1
MM_SHARED = 0x08
MM_UNINIT = 0x10

MM_READ_WRITE = MM_READ | MM_WRITE
MM_READ_EXEC = MM_READ | MM_EXEC
MM_RWX = MM_READ | MM_WRITE | MM_EXEC

pnames = [
        'No Access',
        'Execute',
        'Write',
        'Write/Exec',
        'Read',
        'Read/Exec',
        'Read/Write',
        'RWE',
        ]
# create "shared" versions to pnames
for idx in range(8):
    pnames.append('Shared: ' + pnames[idx])

PAGE_SIZE = 1 << 12
PAGE_NMASK = PAGE_SIZE - 1
PAGE_MASK = ~PAGE_NMASK



import os
import unittest
import vtrace.tests as vt_tests


class VtraceBasicTest(vt_tests.VtraceProcessTest):

    breakpoints = {
        'windows': 'ntdll.NtTerminateProcess',
        'linux': 'libc.exit',
        'freebsd': 'libc.exit',
    }

    def test_vtrace_getregisters(self):
        self.assertTrue(self.trace.getProgramCounter())
        self.assertTrue(self.trace.getStackCounter())
        self.assertTrue(self.trace.getRegisters())
        self.runUntilExit()

    def test_vtrace_setregisters(self):
        pc = self.trace.getProgramCounter()
        sp = self.trace.getStackCounter()
        regs = self.trace.getRegisters()
        self.trace.setStackCounter(sp)
        self.trace.setProgramCounter(pc)
        self.trace.setRegisters(regs)
        self.runUntilExit()

    def test_vtrace_breakpoint(self):
        plat = self.trace.getMeta('Platform')
        symname = self.breakpoints.get(plat)
        if symname is None:
            raise unittest.SkipTest('no platform breakpoint: %s' % plat)

        pycode = 'trace.setMeta("testbphit", 1)'
        bpid = self.trace.addBreakByExpr(symname)
        self.trace.setBreakpointCode(bpid, pycode)
        self.runUntilExit()

        bp = self.trace.getBreakpoint(bpid)
        self.assertTrue(bp.address is not None)
        self.assertTrue(self.trace.getMeta('testbphit'))

    def test_vtrace_exename(self):
        exename = self.trace.getExe()
        self.assertTrue(os.path.isfile(exename))
        self.assertTrue(exename.lower().find('python') != -1)

    def test_vtrace_mmaps(self):
        pymapfound = False
        for va, size, perms, fname in self.trace.getMemoryMaps():
            if fname.lower().find('python') != -1:
                pymapfound = True
                break
        self.assertTrue(pymapfound)


# All of the above "simple" tests should also work in the "exec" case
class VtraceBasicExecTest(VtraceBasicTest, vt_tests.VtraceExecTest):
    breakpoints = {
        'windows': 'ntdll.NtTerminateProcess',
        'linux': 'libc.malloc',
        'freebsd': 'ld._rtld_thread_init',
    }



"""
x86 Support Module
"""
# Copyright (C) 2007 Invisigoth - See LICENSE file for details
import types
import struct
import logging

import vtrace.breakpoints as breakpoints

import envi.archs.i386 as e_i386

# Pre-populating these saves a little processing
# time (important in tight watchpoint loops)
drnames = ["debug%d" % d for d in range(8)]
logger = logging.getLogger(__name__)

dbg_status = "debug6"
dbg_ctrl = "debug7"

dbg_execute = 0
dbg_write = 1
dbg_read_write = 3

dbg_types = {
    "x": dbg_execute,
    "w": dbg_write,
    "rw": dbg_read_write,
}


class i386WatchMixin:
    def __init__(self):
        # Which ones are in use / enabled.
        self.hwdebug = [0, 0, 0, 0]
        # FIXME change this to storing debug0 index and using setRegister()

    def archAddWatchpoint(self, address, size=4, perms="rw"):

        idx = None
        for i in range(4):
            if not self.hwdebug[i]:
                idx = i
                break

        if idx is None:
            raise Exception("ERROR: there...  are... 4... debug registers!")

        pbits = dbg_types.get(perms)
        if pbits is None:
            raise Exception("Unsupported watchpoint perms %s (x86 supports x,w,rw)" % perms)

        if pbits == dbg_execute and size != 1:
            raise Exception("Watchpoint for execute *must* be 1 byte long!")

        if size not in [1, 2, 4]:
            raise Exception("Unsupported watchpoint size %d (x86 supports 1,2,4)" % size)

        ctrl = 0

        self.hwdebug[idx] = address

        ctrl |= 1 << (2*idx)           # Enabled
        mask = ((size-1) << 2) + pbits  # perms and size
        ctrl |= (mask << (16 + (4 * idx)))
        # ctrl |= 0x100 # Local exact (ignored by p6+ for read)

        for tid in self.getThreads().keys():
            ctx = self.getRegisterContext(tid)
            ctrl_orig = ctx.getRegister(e_i386.REG_DEBUG7)
            # logger.debug("debug%d: %.8x debug7: %.8x" % (idx,address,ctrl|ctrl_orig))
            ctx.setRegister(e_i386.REG_DEBUG7, ctrl_orig | ctrl)
            ctx.setRegister(e_i386.REG_DEBUG0 + idx, address)
        return

    def archRemWatchpoint(self, address):
        idx = None
        for i in range(4):
            if self.hwdebug[i] == address:
                idx = i
                break

        if idx is None:
            raise Exception("Watchpoint not found at 0x%.8x" % address)

        self.hwdebug[idx] = 0

        ctrl_disable = ~(1 << (2*idx))      # we are not enabled
        ctrl_disperm = ~(0xf << (16+(4*idx)))  # mask off the rwx stuff
        ctrl_mask = ctrl_disable & ctrl_disperm

        for tid in self.getThreads().keys():
            ctx = self.getRegisterContext(tid)
            ctrl = ctx.getRegister(e_i386.REG_DEBUG7)
            ctrl &= ctrl_mask
            # logger.debug("debug%d: %.8x debug7: %.8x" % (idx,address,ctrl|ctrl_orig))
            ctx.setRegister(e_i386.REG_DEBUG7, ctrl)
            ctx.setRegister(e_i386.REG_DEBUG0 + idx, 0)
        return

    def archCheckWatchpoints(self):
        regs = self.getRegisters()
        status = regs.get(dbg_status)
        # logger.debug("STATUS %.8x" % status)
        if status is None:
            return None
        x = status & 0x0f
        if not x:
            return None

        for i in range(4):
            if (x >> i) & 1:
                return self.hwdebug[i]
        return None


class i386Mixin(e_i386.i386Module, e_i386.i386RegisterContext, i386WatchMixin):

    def __init__(self):
        # Mixin our i386 envi architecture module and register context
        e_i386.i386Module.__init__(self)
        # FIXME tracer base should inherit from RegisterContext and we should
        # just have to load a register definition!
        e_i386.i386RegisterContext.__init__(self)
        i386WatchMixin.__init__(self)

        self.setMeta('Architecture', 'i386')

    def archGetStackTrace(self):
        self.requireAttached()
        current = 0
        sanity = 1000
        frames = []

        # FIXME make these by register index
        # FIXME make these GPREG stuff! (then both are the same)
        ebp = self.getRegisterByName("ebp")
        eip = self.getRegisterByName("eip")
        frames.append((eip, ebp))

        while ebp != 0 and current < sanity:
            try:
                buf = self.readMemory(ebp, 8)
                ebp, eip = struct.unpack("<LL", buf)
                if frames[-1] == (ebp, eip):
                    break
                frames.append((eip, ebp))
                current += 1
            except:
                break

        return frames

    def platformCall(self, address, args, convention=None):
        buf = ""
        finalargs = []
        saved_regs = self.getRegisters()
        sp = self.getStackCounter()
        pc = self.getProgramCounter()

        for arg in args:
            if type(arg) == types.StringType:  # Nicly map strings into mem
                buf = arg+"\x00\x00"+buf    # Pad with a null for convenience
                finalargs.append(sp - len(buf))
            else:
                finalargs.append(arg)

        m = len(buf) % 4
        if m:
            buf = ("\x00" * (4-m)) + buf

        # Args are 
        # finalargs.reverse()
        buf = struct.pack("<%dI" % len(finalargs), *finalargs) + buf

        # Saved EIP is target addr so when we hit the break...
        buf = struct.pack("<I", address) + buf
        # Calc the new stack pointer
        newsp = sp-len(buf)
        # Write the stack buffer in
        self.writeMemory(newsp, buf)
        # Setup the stack pointer
        self.setStackCounter(newsp)
        # Setup the instruction pointer
        self.setProgramCounter(address)
        # Add the magical call-break
        callbreak = breakpoints.CallBreak(address, saved_regs)
        self.addBreakpoint(callbreak)
        # Continue until the CallBreak has been hit
        while not callbreak.endregs:
            self.run()
        return callbreak.endregs



import unittest
import envi.ieee754 as e_float


class FloatingPointTest(unittest.TestCase):
    def test_ieee_754_decode(self):
        # 32 bits
        valu = 0x40e00000
        decoded = e_float.float_decode(valu, 32)

        # intel's "extended" format that uses 80 bits
        valu = 0x3fff8000000000000000
        decoded = e_float.float_decode(valu, 80)
        self.assertEqual(decoded, 1.0)

        valu = 0xbfff8000000000000000
        decoded = e_float.float_decode(valu, 80)
        self.assertEqual(decoded, -1.0)

        valu = 0x4000d49a784bcd1b8afe
        decoded = e_float.float_decode(valu, 80)
        self.assertAlmostEqual(decoded, 3.32192809488736234781)

        valu = 0x4000c90fdaa22168c235
        decoded = e_float.float_decode(valu, 80)
        self.assertAlmostEqual(decoded, 3.14159265358979323851)

    def _encoding_test(self, valu, answers):
        for i, length in enumerate([16, 32, 64, 80, 128]):
            encoded = e_float.float_encode(valu, length)
            try:
                self.assertEqual(encoded, answers[i])
            except:
                self.fail("%s failed to encode properly (produced: 0x%x, test: 0x%x)" % (valu, encoded, answers[i]))

    def test_ieee_754_encode(self):
        pass
        # TODO: double check these and expand them
        #self._encoding_test(1, [0x3c00,
        #                        0x3f800000,
        #                        0x3ff0000000000000,
        #                        0x3fff8000000000000000])

        #self._encoding_test(0.26, [0x3428,
        #                           0x3e851eb8,
        #                           0x3ffd851eb851eb851eb8,
        #                           0x1ffe851eb851eb851ea0L])



import binascii
import unittest

import vstruct
import vstruct.cparse as s_cparse
from vstruct.primitives import *
from vstruct.bitfield import *

from io import BytesIO


class woot(vstruct.VStruct):
    def __init__(self):
        vstruct.VStruct.__init__(self)
        self.lenfield = v_uint8()
        self.strfield = v_str(size=0x20)

    def pcb_lenfield(self):
        self.vsGetField('strfield').vsSetLength(self.lenfield)


class AwesomeTest(s_cparse.CVStruct):
    '''
    struct awesome {
        int x,z;
        char stuff[20];
        int y;
        struct haha {
            int blah;
        } s;
        int *q;
    };
    '''


class VStructTest(unittest.TestCase):

    def test_autoparse(self):
        awe = AwesomeTest()
        awe.vsParse(b'XXXXZZZZhow cool is this?\x00\x00\x00YYYYblahQQQQ')
        self.assertEqual(awe.x, 0x58585858)
        self.assertEqual(awe.z, 0x5A5A5A5A)
        self.assertEqual(awe.stuff, 'how cool is this?')
        self.assertEqual(awe.y, 0x59595959)
        self.assertEqual(awe.s.blah, 0x68616c62)
        self.assertEqual(awe.q, 0x51515151)

    def test_vstruct_basicstruct(self):

        v = vstruct.VStruct()
        v.uint8 = v_uint8(1)
        v.uint16 = v_uint16(2)
        v.uint24 = v_uint24(3)
        v.uint32 = v_uint32(4)
        v.uint64 = v_uint64(5)
        v.vbytes = v_bytes(vbytes=b'ABCD')

        answer = binascii.unhexlify(b'01020003000004000000050000000000000041424344')
        self.assertEqual( v.vsEmit(), answer )


    def test_vstruct_basicreassign(self):
        v = vstruct.VStruct()
        v.uint8 = v_uint8(1)
        v.uint16 = v_uint16(2)
        v.uint24 = v_uint24(3)
        v.uint32 = v_uint32(4)
        v.uint64 = v_uint64(5)
        v.vbytes = v_bytes(vbytes=b'ABCD')

        v.uint8 = 99
        v.uint16 = 100
        v.uint24 = 101
        v.uint32 = 102
        v.uint64 = 103
        v.vbytes = b'\x00\x00\x00\x00'

        answer = binascii.unhexlify('63640065000066000000670000000000000000000000')
        self.assertEqual( v.vsEmit(), answer )


    def test_vstruct_fieldalign(self):
        v = vstruct.VStruct()
        v._vs_field_align = True
        v.uint8 = v_uint8(0x42, bigend=True)
        v.uint16 = v_uint16(0x4243, bigend=True)
        v.uint24 = v_uint24(0x424344, bigend=True)
        v.uint32 = v_uint32(0x42434445, bigend=True)
        v.uint64 = v_uint64(0x4243444546474849, bigend=True)

        answer = binascii.unhexlify('420042430000424344000000424344454243444546474849')
        self.assertEqual( v.vsEmit(), answer )

    def test_vstruct_fixedpartialasign(self):
        v = vstruct.VStruct()
        v.strfield = v_str(size=30)
        v.unifield = v_wstr(size=30)

        v.strfield = 'wootwoot!'
        v.unifield = 'bazbaz'

        answer = binascii.unhexlify('776f6f74776f6f7421000000000000000000000000000000000000000000620061007a00620061007a00000000000000000000000000000000000000')
        self.assertEqual(v.vsEmit(), answer)

    def test_vstruct_lengthcallback(self):

        def updatelen(vs):
            vs.vsGetField('strfield').vsSetLength(vs.lenfield)

        v = vstruct.VStruct()
        v.lenfield = v_uint8(0x30)
        v.strfield = v_str(size=30)
        v.vsAddParseCallback('lenfield', updatelen)

        v.vsParse(b'\x01' + b'A' * 30)
        self.assertEqual( v.vsEmit(), binascii.unhexlify('0141') )


    def test_vstruct_classcallback(self):
        v = woot()
        v.vsParse(b'\x01' + b'A'*30)
        self.assertEqual( v.vsEmit(), binascii.unhexlify('0141') )

    def test_vstruct_parsefd(self):
        v = woot()
        sio = BytesIO((b'\x01' + b'A' * 30))
        v.vsParseFd(sio)
        self.assertEqual( v.vsEmit(), binascii.unhexlify('0141') )

    def test_vstruct_insertfield(self):
        v = woot()
        v.vsInsertField('ifield', v_uint8(), 'strfield')
        v.vsParse(b'\x01BAAAAA')
        self.assertEqual( v.vsEmit(), binascii.unhexlify('014241') )

    def test_vstruct_floats(self):

        v = vstruct.VStruct()
        v.float4 = v_float()
        v.float8 = v_double()

        v.float4 = 99.3
        v.float8 = -400.2

        self.assertEqual(v.vsEmit(), binascii.unhexlify('9a99c64233333333330379c0'))

    def test_vstruct_fastparse(self):
        v = vstruct.VStruct()
        v.x = v_uint8()
        v.y = v_str(size=4)
        v.z = v_uint32()

        v.vsParse(b'BAAAAABCD', fast=True)

        self.assertEqual( v.x, 0x42 )
        self.assertEqual( v.y, 'AAAA' )
        self.assertEqual( v.z, 0x44434241 )

    def test_vstruct_fastparse_bigend(self):
        v = vstruct.VStruct()
        v.x = v_uint8()
        v.y = v_str(size=4)
        v.z = v_uint32(bigend=True)

        v.vsParse(b'BAAAAABCD', fast=True)

        self.assertEqual( v.x, 0x42 )
        self.assertEqual( v.y, 'AAAA' )
        self.assertEqual( v.z, 0x41424344 )

    def test_vstruct_varray(self):
        v = vstruct.VArray( [ v_uint8(i) for i in range(20) ] )
        self.assertEqual(v[2], 2)
        self.assertTrue(v[5] <= 5)
        self.assertTrue(v[5] > 2)
        self.assertTrue(v[19] >= 19)
        v.vsParse(b'A' * 20)
        self.assertEqual( v[2], 0x41 )

    def test_bitfield(self):
        v = VBitField()
        v.vsAddField('w', v_bits(2))
        v.vsAddField('x', v_bits(3))
        v.vsAddField('y', v_bits(3))
        v.vsAddField('z', v_bits(11))
        v.vsAddField('a', v_bits(3))

        v.vsAddField('stuff', v_bits(23))
        v.vsAddField('pad', v_bits(3))
        v.vsAddField('pad2', v_bits(6))
        v.vsAddField('pad3', v_bits(2))

        v.vsParse(b'AAAAAAA')
        self.assertEqual(1, v.w)
        self.assertEqual(0, v.x)
        self.assertEqual(1, v.y)
        self.assertEqual(0x20a, v.z)
        self.assertEqual(0, v.a)
        self.assertEqual(0x282828, v.stuff)
        self.assertEqual(1, v.pad)
        self.assertEqual(16, v.pad2)
        self.assertEqual(1, v.pad3)

        self.assertEqual(b'AAAAAAA', v.vsEmit())

        v.vsParse(b'ABCDEFG')
        self.assertEqual(1, v.w)
        self.assertEqual(0, v.x)
        self.assertEqual(1, v.y)
        self.assertEqual(0x212, v.z)
        self.assertEqual(0, v.a)
        self.assertEqual(0x6888a8, v.stuff)
        self.assertEqual(6, v.pad)
        self.assertEqual(17, v.pad2)
        self.assertEqual(3, v.pad3)

        self.assertEqual(b'ABCDEFG', v.vsEmit())

        v.vsParse(b'zxcvbnm')
        self.assertEqual(1, v.w)
        self.assertEqual(7, v.x)
        self.assertEqual(2, v.y)
        self.assertEqual(0x3c3, v.z)
        self.assertEqual(0, v.a)
        self.assertEqual(0x6ecc4d, v.stuff)
        self.assertEqual(6, v.pad)
        self.assertEqual(0x1b, v.pad2)
        self.assertEqual(1, v.pad3)

        self.assertEqual(b'zxcvbnm', v.vsEmit())

        v.vsParse(b'asdfghj')
        self.assertEqual(1, v.w)
        self.assertEqual(4, v.x)
        self.assertEqual(1, v.y)
        self.assertEqual(0x39b, v.z)
        self.assertEqual(1, v.a)
        self.assertEqual(0xccced, v.stuff)
        self.assertEqual(0, v.pad)
        self.assertEqual(0x1a, v.pad2)
        self.assertEqual(2, v.pad3)

        self.assertEqual(b'asdfghj', v.vsEmit())

    def test_guid(self):
        guid = GUID()
        self.assertEqual(guid.vsGetValue(), '{00000000-0000-0000-0000-000000000000}')

        othr = '{ae03101c-b34a-19cf-9ef3-6d1d62b299a2}'
        guid.vsSetValue(othr)
        self.assertEqual(guid.vsGetValue(), othr)

        gstr = '{59e22249-ffff-0123-f15d-9582e708191e}'
        guid = GUID(guidstr=gstr)
        self.assertEqual(guid.vsGetValue(), gstr)

        bytez = b'I"\xe2Y\xb3\xdb\xa8\x1c\xf1]\x95\x82\xe7\x08\x19\x1e'
        guid.vsParse(bytez)
        self.assertEqual(guid.vsGetValue(), '{59e22249-dbb3-1ca8-f15d-9582e708191e}')
        self.assertEqual(guid.vsEmit(), bytez)

    def test_str(self):
        bytez = b'aquickbrowndog\x00didnotgetfooledbyafox'
        s = v_str(size=100)
        s.vsParse(bytez)
        self.assertEqual(s.vsGetValue(), 'aquickbrowndog')

        s.vsSetLength(128)
        self.assertEqual(s.vsGetValue(), 'aquickbrowndog')
        self.assertEqual(s.vsEmit(), bytez + b'\x00' * (128-len(bytez)))

        s.vsSetLength('6')
        self.assertEqual(s.vsGetValue(), 'aquick')
        self.assertEqual(s.vsEmit(), b'aquick')
        # it doesn't come back
        s.vsSetLength(64)
        self.assertEqual(s.vsGetValue(), 'aquick')
        self.assertEqual(s.vsEmit(), b'aquick' + b'\x00' * 58)

    def test_zstr(self):
        bytez = b'swish\x00\x00swoosh'
        s = v_zstr()
        s.vsParse(bytez)
        self.assertEqual(s.vsGetValue(), 'swish')

        self.assertEqual(s.vsEmit(), b'swish\x00')

    def test_wstr(self):
        bytez = b'a\x00c\x00c\x00e\x00l\x00e\x00r\x00a\x00t\x00o\x00r\x00.\x00d\x00l\x00l\x00\x00\x00'
        s = v_wstr(size=100)
        s.vsParse(bytez)
        self.assertEqual(s.vsGetValue(), "accelerator.dll")
        self.assertEqual(s.vsEmit(), bytez)

    def test_zwstr(self):
        # Very similar to the above, but with slight tweaks to show the use case differences
        bytez = b'a\x00c\x00c\x00e\x00l\x00e\x00r\x00a\x00t\x00o\x00r\x00.\x00d\x00l\x00l\x00\x00\x00y\x00e\x00e\x00t\x00\x00'
        s = v_zwstr()
        s.vsParse(bytez)
        self.assertEqual(s.vsGetValue(), "accelerator.dll")
        self.assertEqual(s.vsEmit(), b'a\x00c\x00c\x00e\x00l\x00e\x00r\x00a\x00t\x00o\x00r\x00.\x00d\x00l\x00l\x00\x00\x00')

    def test_unsigned(self):
        s = v_uint16(value=17)
        byts = s.vsEmit()
        self.assertEqual(b'\x11\x00', byts)

        new = v_uint16()
        new.vsParse(byts)
        self.assertEqual(new.vsEmit(), s.vsEmit())

        # test class that skips by having a real format
        class v_silly(v_number):
            _vs_builder = True
            _vs_length = 5

        num = v_silly(value=1947824, bigend=True)
        byts = num.vsEmit()
        self.assertEqual(b'\x00\x00\x1d\xb8\xb0', byts)

        othr = v_silly(bigend=True)
        othr.vsParse(byts)
        self.assertEqual(othr, num)

    def test_signed(self):
        s = v_int16(value=-1)
        self.assertEqual(b'\xff\xff', s.vsEmit())

        s.vsSetValue(1)
        self.assertEqual(b'\x01\x00', s.vsEmit())

        s = v_int16(value=-1, bigend=True)
        self.assertEqual(b'\xff\xff', s.vsEmit())

        s.vsSetValue(1)
        self.assertEqual(b'\x00\x01', s.vsEmit())

        s = v_int32(value=32)
        self.assertEqual(b'\x20\x00\x00\x00', s.vsEmit())

        s = v_int32(value=-32)
        self.assertEqual(b'\xe0\xff\xff\xff', s.vsEmit())

    def test_repr(self):
        num = v_number(25)
        self.assertEqual(repr(num), '0x19   (25)')
        



import cmd
import logging
import collections

from PyQt5 import QtCore
from PyQt5.QtWidgets import *

import envi.cli
import vtrace.qt
import envi.qt.config

import vdb.qt.memory
import vdb.qt.threads
import vdb.qt.memwrite
import vdb.qt.registers

import vqt.cli
import vqt.main
import vqt.colors
import vqt.qpython
import vqt.hotkeys as vq_hotkeys
import vqt.application as vq_app

from vqt.main import *
from vqt.basics import *
from vqt.common import *
from vtrace.const import *


logger = logging.getLogger(__name__)


class VdbCmdWidget(vqt.cli.VQCli, vtrace.qt.VQTraceNotifier):

    def __init__(self, db, parent=None):

        vqt.cli.VQCli.__init__(self, db, parent)
        vtrace.qt.VQTraceNotifier.__init__(self, trace=db.gui._db_t)
        self._db_t = db.gui._db_t

        self.setAcceptDrops(True)
        self.output.dragEnterEvent = self.dragEnterEvent
        self.output.dropEvent = self.canvasDropEvent

        self.resize(250, 150)

        # set the initial state
        trace = self._db_t
        if trace.isAttached():
            self.notify(NOTIFY_ATTACH, None)
        elif not trace.isAttached():
            self.notify(NOTIFY_DETACH, None)
        elif trace.isRunning():
            self.notify(NOTIFY_CONTINUE, None)
        elif trace.hasExited():
            self.notify(NOTIFY_EXIT, None)
        else:
            self.notify(NOTIFY_CONTINUE, None)

    def getCliLayout(self):
        self.status = QLabel(self)
        self.status.setMinimumWidth(75)
        self.status.setMaximumWidth(75)
        self.status.setFrameStyle(1)
        hbox = HBox(self.status, self.input)
        vbox = VBox(self.output)
        vbox.addLayout(hbox)
        return vbox

    @vqt.main.idlethreadsync
    def notify(self, event, args):

        if not self._db_t.runagain:
            self.status.setText('Paused')
            self.status.setStyleSheet('color:red')

        if event == NOTIFY_ATTACH:
            self.status.setText('Attached')
        elif event in (NOTIFY_BREAK, NOTIFY_SIGNAL):
            self.status.setText('Paused')
            self.status.setStyleSheet('color:red')
        elif event == NOTIFY_CONTINUE:
            self.status.setText('Running')
            self.status.setStyleSheet('color:#00ff00')
        elif event == NOTIFY_DETACH:
            self.status.setText('Detached')
            self.status.setStyleSheet('color:yellow')
        elif event == NOTIFY_EXIT:
            self.status.setText('Exited')
            self.status.setStyleSheet('color:red')

    def returnPressedSlot(self):
        '''
        Override VQCli returnPressedSlot so we submit to the vdb transaction
        queue instead of firing a thread.
        '''
        command = str(self.input.text())
        self.input.clear()
        self.input.setText('')
        self.input.addHistory(command)
        self.output.addText('> %s\n' % command)

        workthread(self.onecmd)(command)

    def onecmd(self, line):
        '''
        Override VQCli onecmd so we can handle exceptions other than
        explicit exiting or keyboard interrupts.
        '''
        lines = line.split('&&')
        try:
            for line in lines:
                line = self.cli.aliascmd(line)
                cmd.Cmd.onecmd(self.cli, line)
        except (KeyboardInterrupt, SystemExit):
            raise

        if self.cli.shutdown.isSet():
            self._emit_quit()

    def isValidDrag(self, event):
        urls = event.mimeData().urls()
        if len(urls) != 1:
            return False

        url = urls[0]
        if not url.isValid() or url.scheme() != 'file':
            return False

        return True

    def dragEnterEvent(self, event):
        if not self.isValidDrag(event):
            event.ignore()
            return

        event.accept()

    def dropEvent(self, event):
        url = event.mimeData().urls()[0]

        self.input.setText('%s"%s"' % (self.input.text(), url.toLocalFile()))

    def canvasDropEvent(self, event):
        url = event.mimeData().urls()[0]

        self.input.setText('exec "%s"' % url.toLocalFile())

class VdbToolBar(vtrace.qt.VQTraceToolBar):
    '''
    Subclass so we get access to the db object not proxied through VdbTrace.
    '''
    def __init__(self, db, trace, parent=None):
        vtrace.qt.VQTraceToolBar.__init__(self, trace, parent=parent)
        self.db = db

    def actAttach(self, *args, **kwargs):
        pid = vtrace.qt.getProcessPid(trace=self.trace)
        if pid is not None:
            workthread(self.trace.attach)(pid)

    @workthread
    def actDetach(self, thing):
        if self.trace.isAttached():
            self.db.do_detach('')

    @workthread
    def actContinue(self, thing):
        self.db.do_go('')

    @workthread
    def actBreak(self, thing):
        self.db.do_break('')

    @workthread
    def actStepi(self, thing):
        self.db.do_stepi('')

    @workthread
    def actStepover(self, thing):
        self.db.do_stepo('')

class VdbWindow(vq_app.VQMainCmdWindow):

    __cli_widget_class__ = VdbCmdWidget

    def __init__(self, db):
        # Gui constructor needs these set...
        self._db = db
        self._db_t = vdb.VdbTrace(db)

        # stores named window names.
        self.namedWindowsCount = collections.Counter()

        db.gui = self # This must go before VQMainCmdWindow
        vq_app.VQMainCmdWindow.__init__(self, 'Vdb', db)

        tbar = VdbToolBar(db, self._db_t, parent=self)
        self.addToolBar(QtCore.Qt.TopToolBarArea, tbar)

        self.vqAddMenuField('&File.&Open', self.menuFileOpen)
        self.vqAddMenuField('&File.&Quit', self.menuFileQuit)
        self.vqAddMenuField('&Edit.&Preferences', self.menuEditPrefs)
        self.vqAddMenuField('&View.&Memory', self.menuViewMemory)
        self.vqAddMenuField('&View.&Memory Maps', self.menuViewMemMaps)
        self.vqAddMenuField('&View.&Threads', self.menuViewThreads)
        self.vqAddMenuField('&View.&Registers', self.menuViewRegisters)
        self.vqAddMenuField('&View.&Memory Write', self.menuViewMemWrite)
        self.vqAddMenuField('&View.&File Descriptors', self.menuViewFileDesc)
        self.vqAddMenuField('&View.&Layouts.&Load', self.menuViewLayoutsLoad)
        self.vqAddMenuField('&View.&Layouts.&Save', self.menuViewLayoutsSave)
        self.vqAddMenuField('&View.&Layouts.&Clear', self.menuViewLayoutsClear)
        self.vqAddMenuField('&Tools.&Python', self.menuToolsPython)

        # Map some default keys
        self.addHotKey('f5','debug:go')
        self.addHotKey('f6','debug:attach')
        self.addHotKey('f7','debug:stepi')
        self.addHotKey('f8','debug:stepover')
        self.addHotKey('ctrl+b','debug:break')
        self.addHotKey('ctrl+p','vdb:view:python')

        self.addHotKey('ctrl+w', 'file:quit')
        self.addHotKeyTarget('file:quit', self.close)

        # Get hotkey overrides
        self.loadHotKeys(self._vq_settings)

        db.registerCmdExtension(self.hotkeys)

        vqtconnect( self.buildMemoryWindow, 'vdb:view:memory' )
        vqtconnect( self.buildMemWriteWindow, 'vdb:view:writemem' )

    def vdbUIEvent(self, event, einfo):
        vqtevent(event,einfo)

    def buildMemoryWindow(self, event, einfo):
        expr, esize, rend = einfo
        dock, view = self.vqBuildDockWidget('VdbMemoryWindow', floating=True)
        view.enviNavGoto(expr, esize, rend)

    def buildMemWriteWindow(self, event, einfo):
        expr, esize = einfo
        dock, view = self.vqBuildDockWidget('VdbMemWriteWindow', floating=True)
        view.enviNavGoto(expr, esize)

    def hotkeys(self, db, line):
        '''
        Manipulate gui hotkeys.

        Usage: hotkeys [target=keyname]
        '''
        argv = envi.cli.splitargs(line)
        for arg in argv:

            if arg.find('=') == -1:
                return db.do_help('hotkeys')

            target,keyname = arg.split('=',1)
            if not self.isHotKeyTarget(target):
                db.vprint('Invalid Hotkey Target: %s' % target)
                return

            db.vprint('Setting: %s = %s' % (target,keyname))

            self.addHotKey(keyname,target)
            self._vq_settings.setValue('hotkey:%s' % target,keyname)

        db.vprint('Hotkeys:')
        lookup = dict([ (targname, keystr) for (keystr,targname) in self.getHotKeys() ])
        targets = self.getHotKeyTargets()
        targets.sort()

        for targname in targets:
            db.vprint('%s: %s' % (targname.ljust(20),lookup.get(targname,'')))

    @vq_hotkeys.hotkey('debug:attach')
    def _hotkey_attach(self):
        trace = self._db.getTrace()
        pid = vtrace.qt.getProcessPid(trace=trace, parent=self)
        workthread(trace.attach)(pid)

    @workthread
    @vq_hotkeys.hotkey('debug:detach')
    def _hotkey_detach(self):
        self._db.do_detach('')

    @workthread
    @vq_hotkeys.hotkey('debug:go')
    def _hotkey_go(self):
        self._db.do_go('')

    @boredthread
    @vq_hotkeys.hotkey('debug:stepi')
    def _hotkey_stepi(self):
        self._db.do_stepi('')

    @boredthread
    @vq_hotkeys.hotkey('debug:stepover')
    def _hotkey_stepover(self):
        self._db.do_stepo('')

    @boredthread
    @vq_hotkeys.hotkey('debug:break')
    def _hotkey_break(self):
        self._db.do_break('')

    def vqInitDockWidgetClasses(self):
        self.vqAddDockWidgetClass(vdb.qt.memory.VdbMemoryWindow, args=(self._db, self._db_t))
        self.vqAddDockWidgetClass(vtrace.qt.VQMemoryMapView, args=(self._db_t, self))
        self.vqAddDockWidgetClass(vtrace.qt.VQFileDescView, args=(self._db_t, self))
        self.vqAddDockWidgetClass(vdb.qt.threads.VdbThreadsWindow, args=(self._db, self._db_t,))
        self.vqAddDockWidgetClass(vqt.qpython.VQPythonView, args=(self._db.getExpressionLocals(),))

        self.vqAddDockWidgetClass(vdb.qt.registers.VdbRegistersWindow, args=(self._db, self._db_t))
        self.vqAddDockWidgetClass(vdb.qt.memwrite.VdbMemWriteWindow, args=(self._db, self._db_t,))

    def menuEditPrefs(self):

        trace = self._db.getTrace()

        config = self._db.config.vdb
        configs = []
        configs.append(('vdb', config))

        arch = trace.getMeta('Architecture').lower()
        platform = trace.getMeta('Platform').lower()

        pconfig = config.getSubConfig( platform, add=False )
        if pconfig is not None:
            configs.append(('vdb:%s' % platform, pconfig))

        aconfig = config.getSubConfig( arch, add=False )
        if aconfig is not None:
            configs.append(('vdb:%s' % arch, aconfig))

        self._cfg_widget = envi.qt.config.EnviConfigTabs( configs )
        self._cfg_widget.show()

    @vq_hotkeys.hotkey('vdb:view:python')
    def menuToolsPython(self):
        self.vqBuildDockWidget('VQPythonView')

    @vq_hotkeys.hotkey('vdb:view:filedesc')
    def menuViewFileDesc(self):
        self.vqBuildDockWidget('VQFileDescView')

    @vq_hotkeys.hotkey('vdb:view:memmaps')
    def menuViewMemMaps(self):
        self.vqBuildDockWidget('VQMemoryMapView')

    @vq_hotkeys.hotkey('vdb:view:memory')
    def menuViewMemory(self):
        self.vqBuildDockWidget('VdbMemoryWindow', floating=True)

    @vq_hotkeys.hotkey('vdb:view:memwrite')
    def menuViewMemWrite(self):
        self.vqBuildDockWidget('VdbMemWriteWindow', floating=True)

    @vq_hotkeys.hotkey('vdb:view:threads')
    def menuViewThreads(self):
        self.vqBuildDockWidget('VdbThreadsWindow', area=QtCore.Qt.RightDockWidgetArea)

    @vq_hotkeys.hotkey('vdb:view:registers')
    def menuViewRegisters(self):
        self.vqBuildDockWidget('VdbRegistersWindow', area=QtCore.Qt.RightDockWidgetArea)

    def menuViewLayoutsLoad(self):
        fname = getOpenFileName(self, 'Load Layout')
        if fname is None:
            return

        self.vqClearDockWidgets()

        settings = QtCore.QSettings(str(fname), QtCore.QSettings.IniFormat)
        self.vqRestoreGuiSettings(settings)

    def menuViewLayoutsSave(self):
        fname = getSaveFileName(self, 'Save Layout')
        if fname is None:
            return

        settings = QtCore.QSettings(str(fname), QtCore.QSettings.IniFormat)
        self.vqSaveGuiSettings(settings)

    def menuViewLayoutsClear(self):
        self.vqClearDockWidgets()

    def menuFileOpen(self, *args, **kwargs):
        fname = str(getOpenFileName(parent=self, caption='File to execute and attach to'))
        if fname != '':
            self._vq_cli.onecmd('exec "%s"' % fname)

    def menuFileQuit(self, *args, **kwargs):
        self.close()

    def closeEvent(self, event):
        try:
            t = self._db.trace

            if t.isAttached():
                if self._db.config.vdb.KillOnQuit:
                    t.kill()

                elif t.isRunning():
                    t.sendBreak()
                    t.detach()

                else:
                    t.detach()

        except Exception as e:
            logger.warning('Error Detaching: %s', e)

        return vq_app.VQMainCmdWindow.closeEvent(self, event)



'''
Home of the code which knows how to actually *draw* a graph which
has been layed out.
'''

class GraphRenderer:
    '''
    Only a "renderer" knows how big each node will end up based on metadata
    in the graph which is specific to the renderer.

    All renderers should be able to set the (x,y) "size" tuple
    '''
    def __init__(self, graph):
        self._vg_graph = graph
        self._vg_layout = None

        self._vg_rendering = False
        self._vg_canvas_width = 0
        self._vg_canvas_height = 0

    def beginRender(self, width, height):
        self._vg_rendering = True
        self._vg_canvas_width = width
        self._vg_canvas_height = height

    def endRender(self):
        self._vg_rendering = False

    def setNodeSizes(self, graph):
        '''
        Calculate the sizes for each node based on graph metadata (or defaults)
        '''
        raise Exception('%s must implement setNodeSizes!' % self.__class__.__name__)

    def renderNode(self, nid, ninfo, xpos, ypos):
        '''
        Render the given node at the specified position.
        '''
        raise Exception('%s must implement renderNode!' % self.__class__.__name__)

    def renderEdge(self, eid, einfo, points):
        '''
        Render an edge in the graph by drawing lines between all the listed
        points (as (x,y) tuples...)
        '''
        raise Exception('%s must implement renderEdge!' % self.__class__.__name__)

    def renderGraph(self):

        graph = self._vg_graph

        width, height = graph.getMeta('size',(800,600))
        self.beginRender(width, height)

        # Render each of the nodes (except ghost nodes...)
        for nid,ninfo in graph.getNodes():
            if ninfo.get('ghost'):
                continue
            xpos, ypos = ninfo.get('position')
            self.renderNode(nid, ninfo, xpos, ypos)

        # Render the edges
        for eid, fromid, toid, einfo in graph.getEdges():
            points = einfo.get('edge_points')
            if points is not None:
                self.renderEdge(eid, einfo, points)

        self.endRender()



import os
import unittest

import cobra.cache as c_cache

class CobraCacheTest(unittest.TestCase):

    #def setUp(self):
        #pass

    #def tearDown(self):
        #pass

    def test_cobra_cache_basic(self):
        cache = c_cache.FixedDepthCache( 5 )
        for i in range(10):
            cache.put(i, 'I: %d' % i)

        self.assertEqual( cache.get(0), None)
        self.assertEqual( cache.get(9), 'I: 9')

    def test_cobra_cache_misscb(self):

        def misscb(x):
            return 'X: %d' % x

        cache = c_cache.FixedDepthCache( 5, misscb=misscb )
        self.assertEqual( cache.get(33), 'X: 33')

    def test_cobra_cache_size(self):

        cache = c_cache.FixedDepthCache( 5 )
        for i in range(10):
            cache.put(i, 'I: %d' % i)

        self.assertEqual( len(cache), 5 )

    def test_cobra_cache_cachefunc(self):

        d = {'hits':0}
        @c_cache.cachefunc(10)
        def cacheme():
            d['hits'] += 1

        cacheme()
        cacheme()

        self.assertEqual( d['hits'], 1 )




import cobra.dcode
import cobra.remoteapp

def shareWorkspace(vw, doref=False):
    daemon = cobra.CobraDaemon('', 0, msgpack=True)
    daemon.fireThread()
    cobra.dcode.enableDcodeServer(daemon=daemon)
    cobra.remoteapp.shareRemoteApp('vivisect.remote.client', appsrv=vw, daemon=daemon)
    return daemon



"""
Watchpoint Objects
"""
# Copyright (C) 2007 Invisigoth - See LICENSE file for details
import envi.const as e_const

from vtrace import *
from vtrace.breakpoints import *

class Watchpoint(Breakpoint):
    """
    The basic "break on access" watchpoint.  Extended from 
    Breakpoints and handled almost exactly the same way...
    """
    def __init__(self, addr, expression=None, size=4, perms="rw"):
        Breakpoint.__init__(self, addr, expression=expression)
        self.wpsize = size
        self.wpperms = perms

    def inittrace(self, trace):
        # No need to get a breakinstr
        pass

    def resolvedaddr(self, trace, addr):
        # We needn't save the memory at our addr...
        pass

    def getName(self):
        bname = Breakpoint.getName(self)
        return "%s (%s %d bytes)" % (bname, self.wpperms, self.wpsize)

    def activate(self, trace):
        trace.requireAttached()
        if not self.active:
            if self.address is not None:
                trace.archAddWatchpoint(self.address, size=self.wpsize, perms=self.wpperms)
                self.active = True
        return self.active

    def deactivate(self, trace):
        trace.requireAttached()
        if self.active:
            trace.archRemWatchpoint(self.address)
            self.active = False
        return self.active

class PageWatchpoint(Watchpoint):
    """
    A special "watchpoint" that uses memory permissions to
    watch for accesses to whole memory maps.  This *requires* OS
    help and only works on platforms which support:
    * platformProtectMemory()
    * signal/exceptions which denote the fault address on SEGV

    NOTE: These *must* be added page aligned
    """
    def __init__(self, addr, expression=None, size=4, watchread=False):
        Watchpoint.__init__(self, addr, expression=expression, size=size, perms='rw')
        self._orig_perms = None
        self._new_perms = e_const.MM_READ
        if watchread:
            self._new_perms = e_const.MM_NONE

    def resolvedaddr(self, trace, addr):
        self._orig_perms = trace.getMemoryMap(addr)[2]

    def notify(self, event, trace):
        pw = trace.getMeta('pagewatch')
        pc = trace.getProgramCounter()
        vaddr,vperm = trace.platformGetMemFault()
        pw.append((pc, vaddr, vperm))
        # Change to/from fastbreak on pagerun...
        self.fastbreak = trace.getMeta('pagerun')

    def getName(self):
        bname = Breakpoint.getName(self)
        return "%s (%s %d bytes)" % (bname, e_mem.reprPerms(self._new_perms), self.wpsize)

    def activate(self, trace):
        #trace.requireNotRunning()
        if not self.active:
            trace.protectMemory(self.address, self.wpsize, self._new_perms)
            self.active = True
        return self.active

    def deactivate(self, trace):
        #trace.requireNotRunning()
        if self.active:
            trace.protectMemory(self.address, self.wpsize, self._orig_perms)
            self.active = False
        return self.active




# Fat Defines...
FAT_MAGIC = 0xcafebabe
FAT_CIGAM = 0xbebafeca   # NXSwapLong(FAT_MAGIC)
FAT_MAGIC_64 = 0xcafebabf
FAT_CIGAM_64 = 0xbfbafeca

FAT_DEFINES = (FAT_MAGIC, FAT_CIGAM, FAT_MAGIC_64, FAT_CIGAM_64)
FAT_64 = (FAT_MAGIC_64, FAT_CIGAM_64)

MH_MAGIC                  = 0xfeedface #  the mach magic number 
MH_CIGAM                  = 0xcefaedfe #  NXSwapInt(MH_MAGIC) 
MH_MAGIC_64               = 0xfeedfacf #  the 64-bit mach magic number 
MH_CIGAM_64               = 0xcffaedfe #  NXSwapInt(MH_MAGIC_64) 
MH_OBJECT                 = 0x1 #  relocatable object file 
MH_EXECUTE                = 0x2 #  demand paged executable file 
MH_FVMLIB                 = 0x3 #  fixed VM shared library file 
MH_CORE                   = 0x4 #  core file 
MH_PRELOAD                = 0x5 #  preloaded executable file 
MH_DYLIB                  = 0x6 #  dynamically bound shared library 
MH_DYLINKER               = 0x7 #  dynamic link editor 
MH_BUNDLE                 = 0x8 #  dynamically bound bundle file 
MH_DYLIB_STUB             = 0x9 #  shared library stub for static 
MH_DSYM                   = 0xa #  companion file with only debug 
MH_NOUNDEFS               = 0x1 #  the object file has no undefinedreferences 
MH_INCRLINK               = 0x2 #  the object file is the output of anincremental link against a base fileand can't be link edited again 
MH_DYLDLINK               = 0x4 #  the object file is input for thedynamic linker and can't be staticlylink edited again 
MH_BINDATLOAD             = 0x8 #  the object file's undefinedreferences are bound by the dynamic linker when loaded. 
MH_PREBOUND               = 0x10 #  the file has its dynamic undefinedreferences prebound. 
MH_SPLIT_SEGS             = 0x20 #  the file has its read-only andread-write segments split 
MH_LAZY_INIT              = 0x40 #  the shared library init routine isto be run lazily via catching memoryfaults to its writeable segments(obsolete) 
MH_TWOLEVEL               = 0x80 #  the image is using two-level namespace bindings 
MH_FORCE_FLAT             = 0x100 #  the executable is forcing all imagesto use flat name space bindings 
MH_NOMULTIDEFS            = 0x200 #  this umbrella guarantees no multipledefintions of symbols in itssub-images so the two-level namespacehints can always be used. 
MH_NOFIXPREBINDING        = 0x400 #  do not have dyld notify theprebinding agent about thisexecutable 
MH_PREBINDABLE            = 0x800 #  the binary is not prebound but canhave its prebinding redone. only usedwhen MH_PREBOUND is not set. 
MH_ALLMODSBOUND           = 0x1000 #  this binary binds toall two-level namespace modules ofits dependent libraries. only usedwhen MH_PREBINDABLE and MH_TWOLEVELare both set. 
MH_CANONICAL              = 0x4000 #  the binary has been canonicalizedvia the unprebind operation 
MH_WEAK_DEFINES           = 0x8000 #  the final linked image containsexternal weak symbols 
MH_BINDS_TO_WEAK          = 0x10000 #  the final linked image usesweak symbols 
MH_ROOT_SAFE              = 0x40000 #  When this bit is set, the binarydeclares it is safe for use inprocesses with uid zero 
MH_SETUID_SAFE            = 0x80000 #  When this bit is set, the binarydeclares it is safe for use inprocesses when issetugid() is true 
MH_NO_REEXPORTED_DYLIBS   = 0x100000 #  When this bit is set on a dylib,the static linker does not need toexamine dependent dylibs to seeif any are re-exported 
MH_PIE                    = 0x200000 #  When this bit is set, the OS willload the main executable at arandom address. Only used inMH_EXECUTE filetypes. 

# Constants for the cmd field of all load commands, the type
LC_REQ_DYLD               = 0x80000000 #  When this bit is set, the OS will load the main executable at a random address. Only used in MH_EXECUTE filetypes. 
LC_SEGMENT                = 0x1 #  segment of this file to be mapped 
LC_SYMTAB                 = 0x2 #  link-edit stab symbol table info 
LC_SYMSEG                 = 0x3 #  link-edit gdb symbol table info (obsolete) 
LC_THREAD                 = 0x4 #  thread 
LC_UNIXTHREAD             = 0x5 #  unix thread (includes a stack) 
LC_LOADFVMLIB             = 0x6 #  load a specified fixed VM shared library 
LC_IDFVMLIB               = 0x7 #  fixed VM shared library identification 
LC_IDENT                  = 0x8 #  object identification info (obsolete) 
LC_FVMFILE                = 0x9 #  fixed VM file inclusion (internal use) 
LC_PREPAGE                = 0xa #  prepage command (internal use) 
LC_DYSYMTAB               = 0xb #  dynamic link-edit symbol table info 
LC_LOAD_DYLIB             = 0xc #  load a dynamically linked shared library 
LC_ID_DYLIB               = 0xd #  dynamically linked shared lib ident 
LC_LOAD_DYLINKER          = 0xe #  load a dynamic linker 
LC_ID_DYLINKER            = 0xf #  dynamic linker identification 
LC_PREBOUND_DYLIB         = 0x10 #  modules prebound for a dynamically linked shared library
LC_ROUTINES               = 0x11 #  image routines 
LC_SUB_FRAMEWORK          = 0x12 #  sub framework 
LC_SUB_UMBRELLA           = 0x13 #  sub umbrella 
LC_SUB_CLIENT             = 0x14 #  sub client 
LC_SUB_LIBRARY            = 0x15 #  sub library 
LC_TWOLEVEL_HINTS         = 0x16 #  two-level namespace lookup hints 
LC_PREBIND_CKSUM          = 0x17 #  prebind checksum 
LC_LOAD_WEAK_DYLIB        = (0x18 | LC_REQ_DYLD)  # load a dynamically linked shared library that is allowed to be missing (all symbols are weak imported).
LC_SEGMENT_64             = 0x19 #  64-bit segment of this file to bemapped 
LC_ROUTINES_64            = 0x1a #  64-bit image routines 
LC_UUID                   = 0x1b #  the uuid 
LC_RPATH                  = (0x1c | LC_REQ_DYLD)  # runpath additions
LC_CODE_SIGNATURE         = 0x1d #  local of code signature 
LC_SEGMENT_SPLIT_INFO     = 0x1e #  local of info to split segments 
LC_REEXPORT_DYLIB         = (0x1f | LC_REQ_DYLD)  # load and re-export dylib
LC_LAZY_LOAD_DYLIB        = 0x20 #  delay load of dylib until first use 
LC_ENCRYPTION_INFO        = 0x21 #  encrypted segment information 
LC_DYLD_INFO              = 0x22 #  compressed dyld information
LC_DYLD_INFO_ONLY         = (0x22 | LC_REQ_DYLD)  # compressed dyld information only
LC_LOAD_UPWARD_DYLIB      = (0x23 | LC_REQ_DYLD)  # load upward dylib
LC_VERSION_MIN_MACOSX     = 0x24  # build for MacOSX min OS version
LC_VERSION_MIN_IPHONEOS   = 0x25  # build for iPhoneOS min OS version
LC_FUNCTION_STARTS        = 0x26  # compressed table of function start addresses
LC_DYLD_ENVIRONMENT       = 0x27  # string for dyld to treat like environment variable
LC_MAIN                   = (0x28 | LC_REQ_DYLD)  # replacement for LC_UNIXTHREAD
LC_DATA_IN_CODE           = 0x29  # table of non-instructions in __text
LC_SOURCE_VERSION         = 0x2A  # source version used to build binary
LC_DYLIB_CODE_SIGN_DRS    = 0x2B  # Code signing DRs copied from linked dylibs

SG_HIGHVM                 = 0x1 #  the file contents for this segment is for the high part of the VM space, the low part is zero filled (for stacks in core files) 
SG_FVMLIB                 = 0x2 #  this segment is the VM that is allocated by a fixed VM library, for overlap checking in the link editor 
SG_NORELOC                = 0x4 #  this segment has nothing that was relocated in it and nothing relocated to it, that is it maybe safely replaced without relocation
SG_PROTECTED_VERSION_1    = 0x8 #  This segment is protected. If the segment starts at file offset 0, the first page of the segment is not protected. All other pages of the segment are protected. 


SECTION_TYPE              = 0x000000ff #  256 section types 
SECTION_ATTRIBUTES        = 0xffffff00 #  24 section attributes 
S_REGULAR                 = 0x0 #  regular section 
S_ZEROFILL                = 0x1 #  zero fill on demand section 
S_CSTRING_LITERALS        = 0x2 #  section with only literal C strings
S_4BYTE_LITERALS          = 0x3 #  section with only 4 byte literals 
S_8BYTE_LITERALS          = 0x4 #  section with only 8 byte literals 
S_LITERAL_POINTERS        = 0x5 #  section with only pointers to literals
S_NON_LAZY_SYMBOL_POINTERS = 0x6 #  section with only non-lazy symbol pointers 
S_LAZY_SYMBOL_POINTERS    = 0x7 #  section with only lazy symbol pointers 
S_SYMBOL_STUBS            = 0x8 #  section with only symbol stubs, byte size of stub in the reserved2 field 
S_MOD_INIT_FUNC_POINTERS  = 0x9 #  section with only function pointers for initialization
S_MOD_TERM_FUNC_POINTERS  = 0xa #  section with only function pointers for termination 
S_COALESCED               = 0xb #  section contains symbols that are to be coalesced 
S_GB_ZEROFILL             = 0xc #  zero fill on demand section (that can be larger than 4gigabytes) 
S_INTERPOSING             = 0xd #  section with only pairs offunction pointers for interposing 
S_16BYTE_LITERALS         = 0xe #  section with only 16 byte literals 
S_DTRACE_DOF              = 0xf #  section contains DTrace Object Format 
S_LAZY_DYLIB_SYMBOL_POINTERS = 0x10 #  section with only lazysymbol pointers to lazyloaded dylibs 
SECTION_ATTRIBUTES_USR    = 0xff000000 #  User setable attributes 
S_ATTR_PURE_INSTRUCTIONS  = 0x80000000 #  section contains only true machine instructions 
S_ATTR_NO_TOC             = 0x40000000 #  section contains coalesced symbols that are not to bein a ranlib table of contents 
S_ATTR_STRIP_STATIC_SYMS  = 0x20000000 #  ok to strip static symbolsin this section in files with the MH_DYLDLINK flag 
S_ATTR_NO_DEAD_STRIP      = 0x10000000 #  no dead stripping 
S_ATTR_LIVE_SUPPORT       = 0x08000000 #  blocks are live if they reference live blocks 
S_ATTR_SELF_MODIFYING_CODE = 0x04000000 #  Used with i386 code stubs written on by dyld 
S_ATTR_DEBUG              = 0x02000000 #  a debug section 
SECTION_ATTRIBUTES_SYS    = 0x00ffff00 #  system setable attributes 
S_ATTR_SOME_INSTRUCTIONS  = 0x00000400 #  section contains some machine instructions 
S_ATTR_EXT_RELOC          = 0x00000200 #  section has external relocation entries 
S_ATTR_LOC_RELOC          = 0x00000100 #  section has local relocation entries 
INDIRECT_SYMBOL_LOCAL     = 0x80000000 #  section has local relocation entries 
INDIRECT_SYMBOL_ABS       = 0x40000000 #  section has local relocation entries 

CPU_ARCH_MASK = 0xff000000
CPU_ARCH_ABI64 = 0x01000000
CPU_ARCH_ABI64_32 = 0x02000000

CPU_TYPE_ANY        = -1
CPU_TYPE_VAX        = 1
CPU_TYPE_MC680      = 6
CPU_TYPE_X86        = 7
CPU_TYPE_X86_64     = CPU_TYPE_X86 | CPU_ARCH_ABI64
CPU_TYPE_MIPS       = 8
CPU_TYPE_MC98000    = 10
CPU_TYPE_HPPA       = 11
CPU_TYPE_ARM        = 12
CPU_TYPE_ARM64      = CPU_TYPE_ARM | CPU_ARCH_ABI64
CPU_TYPE_ARM64_32   = CPU_TYPE_ARM | CPU_ARCH_ABI64_32
CPU_TYPE_MC88000    = 13
CPU_TYPE_SPARC      = 14
CPU_TYPE_I860       = 15
CPU_TYPE_ALPHA      = 16
CPU_TYPE_POWERPC    = 18
CPU_TYPE_POWERPC64  = CPU_TYPE_POWERPC | CPU_ARCH_ABI64

mach_cpu_names = {
    CPU_TYPE_VAX        : 'vax',
    CPU_TYPE_MC680      : 'mc680',
    CPU_TYPE_X86        : 'i386',
    CPU_TYPE_X86_64     : 'amd64',
    CPU_TYPE_MIPS       : 'mips',
    CPU_TYPE_MC98000    : 'mc98000',
    CPU_TYPE_HPPA       : 'hppa',
    CPU_TYPE_ARM        : 'arm',
    CPU_TYPE_ARM64      : 'arm64',
    CPU_TYPE_ARM64_32   : 'arm64_32',
    CPU_TYPE_MC88000    : 'mc88000',
    CPU_TYPE_SPARC      : 'sparc',
    CPU_TYPE_I860       : 'i860',
    CPU_TYPE_ALPHA      : 'alpha',
    CPU_TYPE_POWERPC    : 'powerpc',
    CPU_TYPE_POWERPC64  : 'powerpc64',
}

# Symbol types
N_GSYM      = 0x20      # global symbol:                name,,NO_SECT,type,0
N_FNAME     = 0x22      # procedure name (f77 kludge):  name,,NO_SECT,0,0
N_FUN       = 0x24      # procedure:                    name,,n_sect,linenumber,address
N_STSYM     = 0x26      # static symbol:                name,,n_sect,type,address
N_LCSYM     = 0x28      # .lcomm symbol:                name,,n_sect,type,address
N_BNSYM     = 0x2e      # begin nsect sym:              0,,n_sect,0,address
N_PC        = 0x30      # global pascal symbol:         name,,NO_SECT,subtype,line
N_OPT       = 0x3c      # emitted with gcc2_compile and in gcc source
N_RSYM      = 0x40      # register sym:                 name,,NO_SECT,type,register
N_SLINE     = 0x44      # src line:                     0,,n_sect,linenumber,address
N_ENSYM     = 0x4e      # end nsect sym:                0,,n_sect,0,address
N_SSYM      = 0x60      # struct elt:                   name,,NO_SECT,type,struct_offset
N_SO        = 0x64      # source file name:             name,,n_sect,0,address
N_LSYM      = 0x80      # local sym:                    name,,NO_SECT,type,offset
N_BINCL     = 0x82      # include file beginning:       name,,NO_SECT,0,sum
N_SOL       = 0x84      # #included file name:          name,,n_sect,0,address
N_PARAMS    = 0x86      # compiler parameters:          name,,NO_SECT,0,0
N_VERSION   = 0x88      # compiler version:             name,,NO_SECT,0,0
N_OLEVEL    = 0x8A      # compiler -O level:            name,,NO_SECT,0,0
N_PSYM      = 0xA0      # parameter:                    name,,NO_SECT,type,offset
N_EINCL     = 0xA2      # include file end:             name,,NO_SECT,0,0
N_ENTRY     = 0xA4      # alternate entry:              name,,n_sect,linenumber,address
N_LBRAC     = 0xC0      # left bracket:                 0,,NO_SECT,nesting level,address
N_EXCL      = 0xC2      # deleted include file:         name,,NO_SECT,0,sum
N_RBRAC     = 0xE0      # right bracket:                0,,NO_SECT,nesting level,address
N_BCOMM     = 0xE2      # begin common:                 name,,NO_SECT,0,0
N_ECOMM     = 0xE4      # end common:                   name,,n_sect,0,0
N_ECOML     = 0xE8      # end common (local name):      0,,n_sect,0,address
N_LENG      = 0xFE      # second stab entry with length information

# The n_type field really contains four fields:
#	unsigned char N_STAB:3,
#		      N_PEXT:1,
#		      N_TYPE:3,
#		      N_EXT:1;

N_STAB  = 0xe0  # if any of these bits set, a symbolic debugging entry
N_PEXT  = 0x10  # private external symbol bit
N_TYPE  = 0x0e  # mask for the type bits
N_EXT   = 0x01  # external symbol bit, set for external symbols

# Values for N_TYPE bits of the n_type field.
N_UNDF   = 0x0 # undefined, n_sect == NO_SECT
N_ABS    = 0x2 # absolute, n_sect == NO_SECT
N_SECT   = 0xe # defined in section number n_sect
N_PBUD   = 0xc # prebound undefined (defined in a dylib)
N_INDR   = 0xa # indirect

ENDIAN_LSB = 0
ENDIAN_MSB = 1



DICE_KIND_DATA              = 0x0001  # L$start$data$...  label
DICE_KIND_JUMP_TABLE8       = 0x0002  # L$start$jt8$...   label
DICE_KIND_JUMP_TABLE16      = 0x0003  # L$start$jt16$...  label
DICE_KIND_JUMP_TABLE32      = 0x0004  # L$start$jt32$...  label
DICE_KIND_ABS_JUMP_TABLE32  = 0x0005  # L$start$jta32$... label

CSMAGIC_REQUIREMENT = 0xfade0c00  # single Requirement blob
CSMAGIC_REQUIREMENTS = 0xfade0c01 # Requirements vector (internal requirements)
CSMAGIC_CODEDIRECTORY = 0xfade0c02      # CodeDirectory blob
CSMAGIC_EMBEDDED_SIGNATURE = 0xfade0cc0 # embedded form of signature data
CSMAGIC_EMBEDDED_SIGNATURE_OLD = 0xfade0b02 # XXX */
CSMAGIC_EMBEDDED_ENTITLEMENTS = 0xfade7171  # embedded entitlements
CSMAGIC_DETACHED_SIGNATURE = 0xfade0cc1 # multi-arch collection of embedded signatures
CSMAGIC_BLOBWRAPPER = 0xfade0b01 # CMS Signature, among other things

CS_SUPPORTSSCATTER = 0x20100
CS_SUPPORTSTEAMID = 0x20200
CS_SUPPORTSCODELIMIT64 = 0x20300
CS_SUPPORTSEXECSEG = 0x20400
CS_SUPPORTSRUNTIME = 0x20500
CS_SUPPORTSLINKAGE = 0x20600

CSSLOT_CODEDIRECTORY = 0 # slot index for CodeDirectory
CSSLOT_INFOSLOT = 1
CSSLOT_REQUIREMENTS = 2
CSSLOT_RESOURCEDIR = 3
CSSLOT_APPLICATION = 4
CSSLOT_ENTITLEMENTS = 5
CSSLOT_REP_SPECIFIC = 6
CSSLOT_ENTITLEMENTS_DER = 7

CSSLOT_ALTERNATE_CODEDIRECTORIES =  0x1000
CSSLOT_ALTERNATE_CODEDIRECTORIES_MAX = 5

CSSLOT_CMS_SIGNATURE = 0x10000
CSSLOT_IDENTIFICATIONSLOT = 0x10001
CSSLOT_TICKETSLOT = 0x10002

CSSLOT_ALTERNATE_CODEDIRECTORIES = 0x1000 # first alternate CodeDirectory, if any
CSSLOT_ALTERNATE_CODEDIRECTORY_MAX = 5 # max number of alternate CD slots
CSSLOT_ALTERNATE_CODEDIRECTORY_LIMIT = CSSLOT_ALTERNATE_CODEDIRECTORIES + CSSLOT_ALTERNATE_CODEDIRECTORY_MAX # one past the last

CSSLOT_SIGNATURESLOT = 0x10000 # CMS Signature
CSSLOT_IDENTIFICATIONSLOT = 0x10001
CSSLOT_TICKETSLOT = 0x10002

CSTYPE_INDEX_REQUIREMENTS = 0x00000002 # compat with amf
CSTYPE_INDEX_ENTITLEMENTS = 0x00000005 # compat with amfi

CS_HASHTYPE_SHA1 = 1
CS_HASHTYPE_SHA256 = 2
CS_HASHTYPE_SHA256_TRUNCATED = 3
CS_HASHTYPE_SHA384 = 4
CS_HASHTYPE_SHA512 = 5

CS_SHA1_LEN = 20
CS_SHA256_LEN = 32
CS_SHA256_TRUNCATED_LEN = 20

CS_CDHASH_LEN = 20    # always - larger hashes are truncated
CS_HASH_MAX_SIZE = 48 # max size of the hash we'll support

# From LLVM:
# Currently only to support Legacy VPN plugins, and Mac App Store
# but intended to replace all the various platform code, dev code etc. bits.
CS_SIGNER_TYPE_UNKNOWN = 0
CS_SIGNER_TYPE_LEGACYVPN = 5
CS_SIGNER_TYPE_MAC_APP_STORE = 6

CS_SUPPL_SIGNER_TYPE_UNKNOWN = 0
CS_SUPPL_SIGNER_TYPE_TRUSTCACHE = 7
CS_SUPPL_SIGNER_TYPE_LOCAL = 8



import binascii

import vstruct
from vstruct.primitives import *

class IMAGE_BASE_RELOCATION(vstruct.VStruct):
    def __init__(self):
        vstruct.VStruct.__init__(self)
        self.VirtualAddress = v_uint32()
        self.SizeOfBlock    = v_uint32()

class IMAGE_DATA_DIRECTORY(vstruct.VStruct):
    def __init__(self):
        vstruct.VStruct.__init__(self)
        self.VirtualAddress = v_uint32()
        self.Size           = v_uint32()

class IMAGE_DOS_HEADER(vstruct.VStruct):
    def __init__(self):
        vstruct.VStruct.__init__(self)
        self.e_magic    = v_uint16()
        self.e_cblp     = v_uint16()
        self.e_cp       = v_uint16()
        self.e_crlc     = v_uint16()
        self.e_cparhdr  = v_uint16()
        self.e_minalloc = v_uint16()
        self.e_maxalloc = v_uint16()
        self.e_ss       = v_uint16()
        self.e_sp       = v_uint16()
        self.e_csum     = v_uint16()
        self.e_ip       = v_uint16()
        self.e_cs       = v_uint16()
        self.e_lfarlc   = v_uint16()
        self.e_ovno     = v_uint16()
        self.e_res      = vstruct.VArray([v_uint16() for i in range(4)])
        self.e_oemid    = v_uint16()
        self.e_oeminfo  = v_uint16()
        self.e_res2     = vstruct.VArray([v_uint16() for i in range(10)])
        self.e_lfanew   = v_uint32()

class IMAGE_EXPORT_DIRECTORY(vstruct.VStruct):
    def __init__(self):
        vstruct.VStruct.__init__(self)
        self.Characteristics    = v_uint32()
        self.TimeDateStamp      = v_uint32()
        self.MajorVersion       = v_uint16()
        self.MinorVersion       = v_uint16()
        self.Name               = v_uint32()
        self.Base               = v_uint32()
        self.NumberOfFunctions  = v_uint32()
        self.NumberOfNames      = v_uint32()
        self.AddressOfFunctions = v_uint32()
        self.AddressOfNames     = v_uint32()
        self.AddressOfOrdinals  = v_uint32()

class IMAGE_DEBUG_DIRECTORY(vstruct.VStruct):
    def __init__(self):
      vstruct.VStruct.__init__(self)
      self.Characteristics = v_uint32()
      self.TimeDateStamp   = v_uint32()
      self.MajorVersion    = v_uint16()
      self.MinorVersion    = v_uint16()
      self.Type            = v_uint32()
      self.SizeOfData      = v_uint32()
      self.AddressOfRawData= v_uint32()
      self.PointerToRawData= v_uint32()

class CV_INFO_PDB70(vstruct.VStruct):
    def __init__(self):
        vstruct.VStruct.__init__(self)
        self.CvSignature    = v_uint32()
        self.Signature      = GUID()
        self.Age            = v_uint32()
        self.PdbFileName    = v_str(260)

    def vsParse(self, bytez, offset=0):
        bsize = len(bytez) - offset
        self.vsGetField('PdbFileName').vsSetLength( bsize - 24 )
        return vstruct.VStruct.vsParse(self, bytez, offset=offset)

class IMAGE_FILE_HEADER(vstruct.VStruct):
    def __init__(self):
        vstruct.VStruct.__init__(self)
        self.Machine              = v_uint16()
        self.NumberOfSections     = v_uint16()
        self.TimeDateStamp        = v_uint32()
        self.PointerToSymbolTable = v_uint32()
        self.NumberOfSymbols      = v_uint32()
        self.SizeOfOptionalHeader = v_uint16()
        self.Characteristics      = v_uint16()

class IMAGE_IMPORT_DIRECTORY(vstruct.VStruct):
    def __init__(self):
        vstruct.VStruct.__init__(self)
        self.OriginalFirstThunk = v_uint32()
        self.TimeDateStamp      = v_uint32()
        self.ForwarderChain     = v_uint32()
        self.Name               = v_uint32()
        self.FirstThunk         = v_uint32()

# https://docs.microsoft.com/en-us/cpp/build/reference/structure-and-constant-definitions?view=vs-2019
class IMAGE_DELAY_IMPORT_DIRECTORY(vstruct.VStruct):
    def __init__(self):
        vstruct.VStruct.__init__(self)
        self.grAttrs      = v_uint32()
        self.rvaDLLName   = v_uint32()
        self.rvaHmod      = v_uint32()
        self.rvaIAT       = v_uint32()
        self.rvaINT       = v_uint32()
        self.rvaBoundIAT  = v_uint32()
        self.rvaUnloadIAT = v_uint32()
        self.dwTimeStamp  = v_uint32()

class IMAGE_IMPORT_BY_NAME(vstruct.VStruct):
    def __init__(self, namelen=128):
        vstruct.VStruct.__init__(self)
        self.Hint   = v_uint16()
        self.Name   = v_str(size=namelen)

class IMAGE_LOAD_CONFIG_DIRECTORY(vstruct.VStruct):
    def __init__(self):
        vstruct.VStruct.__init__(self)
        self.Size                          = v_uint32()
        self.TimeDateStamp                 = v_uint32()
        self.MajorVersion                  = v_uint16()
        self.MinorVersion                  = v_uint16()
        self.GlobalFlagsClear              = v_uint32()
        self.GlobalFlagsSet                = v_uint32()
        self.CriticalSectionDefaultTimeout = v_uint32()
        self.DeCommitFreeBlockThreshold    = v_uint32()
        self.DeCommitTotalFreeThreshold    = v_uint32()
        self.LockPrefixTable               = v_uint32()
        self.MaximumAllocationSize         = v_uint32()
        self.VirtualMemoryThreshold        = v_uint32()
        self.ProcessHeapFlags              = v_uint32()
        self.ProcessAffinityMask           = v_uint32()
        self.CSDVersion                    = v_uint16()
        self.Reserved1                     = v_uint16()
        self.EditList                      = v_uint32()
        self.SecurityCookie                = v_uint32()
        self.SEHandlerTable                = v_uint32()
        self.SEHandlerCount                = v_uint32()

class IMAGE_NT_HEADERS(vstruct.VStruct):
    def __init__(self):
        vstruct.VStruct.__init__(self)
        self.Signature      = v_bytes(4)
        self.FileHeader     = IMAGE_FILE_HEADER()
        self.OptionalHeader = IMAGE_OPTIONAL_HEADER()

class IMAGE_NT_HEADERS64(vstruct.VStruct):
    def __init__(self):
        vstruct.VStruct.__init__(self)
        self.Signature      = v_bytes(4)
        self.FileHeader     = IMAGE_FILE_HEADER()
        self.OptionalHeader = IMAGE_OPTIONAL_HEADER64()

class IMAGE_OPTIONAL_HEADER(vstruct.VStruct):
    def __init__(self):
        vstruct.VStruct.__init__(self)
        self.Magic                       = v_bytes(2)
        self.MajorLinkerVersion          = v_uint8()
        self.MinorLinkerVersion          = v_uint8()
        self.SizeOfCode                  = v_uint32()
        self.SizeOfInitializedData       = v_uint32()
        self.SizeOfUninitializedData     = v_uint32()
        self.AddressOfEntryPoint         = v_uint32()
        self.BaseOfCode                  = v_uint32()
        self.BaseOfData                  = v_uint32()
        self.ImageBase                   = v_uint32()
        self.SectionAlignment            = v_uint32()
        self.FileAlignment               = v_uint32()
        self.MajorOperatingSystemVersion = v_uint16()
        self.MinorOperatingSystemVersion = v_uint16()
        self.MajorImageVersion           = v_uint16()
        self.MinorImageVersion           = v_uint16()
        self.MajorSubsystemVersion       = v_uint16()
        self.MinorSubsystemVersion       = v_uint16()
        self.Win32VersionValue           = v_uint32()
        self.SizeOfImage                 = v_uint32()
        self.SizeOfHeaders               = v_uint32()
        self.CheckSum                    = v_uint32()
        self.Subsystem                   = v_uint16()
        self.DllCharacteristics          = v_uint16()
        self.SizeOfStackReserve          = v_uint32()
        self.SizeOfStackCommit           = v_uint32()
        self.SizeOfHeapReserve           = v_uint32()
        self.SizeOfHeapCommit            = v_uint32()
        self.LoaderFlags                 = v_uint32()
        self.NumberOfRvaAndSizes         = v_uint32()
        self.DataDirectory               = vstruct.VArray([IMAGE_DATA_DIRECTORY() for i in range(16)])

class IMAGE_OPTIONAL_HEADER64(vstruct.VStruct):
    def __init__(self):
        vstruct.VStruct.__init__(self)
        self.Magic                       = v_bytes(2)
        self.MajorLinkerVersion          = v_uint8()
        self.MinorLinkerVersion          = v_uint8()
        self.SizeOfCode                  = v_uint32()
        self.SizeOfInitializedData       = v_uint32()
        self.SizeOfUninitializedData     = v_uint32()
        self.AddressOfEntryPoint         = v_uint32()
        self.BaseOfCode                  = v_uint32()
        self.ImageBase                   = v_uint64()
        self.SectionAlignment            = v_uint32()
        self.FileAlignment               = v_uint32()
        self.MajorOperatingSystemVersion = v_uint16()
        self.MinorOperatingSystemVersion = v_uint16()
        self.MajorImageVersion           = v_uint16()
        self.MinorImageVersion           = v_uint16()
        self.MajorSubsystemVersion       = v_uint16()
        self.MinorSubsystemVersion       = v_uint16()
        self.Win32VersionValue           = v_uint32()
        self.SizeOfImage                 = v_uint32()
        self.SizeOfHeaders               = v_uint32()
        self.CheckSum                    = v_uint32()
        self.Subsystem                   = v_uint16()
        self.DllCharacteristics          = v_uint16()
        self.SizeOfStackReserve          = v_uint64()
        self.SizeOfStackCommit           = v_uint64()
        self.SizeOfHeapReserve           = v_uint64()
        self.SizeOfHeapCommit            = v_uint64()
        self.LoaderFlags                 = v_uint32()
        self.NumberOfRvaAndSizes         = v_uint32()
        self.DataDirectory               = vstruct.VArray([IMAGE_DATA_DIRECTORY() for i in range(16)])

class IMAGE_RESOURCE_DIRECTORY(vstruct.VStruct):
    def __init__(self):
        vstruct.VStruct.__init__(self)
        self.Characteristics      = v_uint32()
        self.TimeDateStamp        = v_uint32()
        self.MajorVersion         = v_uint16()
        self.MinorVersion         = v_uint16()
        self.NumberOfNamedEntries = v_uint16()
        self.NumberOfIdEntries    = v_uint16()

class IMAGE_RESOURCE_DIRECTORY_ENTRY(vstruct.VStruct):
    def __init__(self):
        vstruct.VStruct.__init__(self)
        self.Name           = v_uint32()
        self.OffsetToData   = v_uint32()

class IMAGE_RESOURCE_DATA_ENTRY(vstruct.VStruct):
    def __init__(self):
        vstruct.VStruct.__init__(self)
        self.OffsetToData   = v_uint32()
        self.Size           = v_uint32()
        self.CodePage       = v_uint32()
        self.Reserved       = v_uint32()

class VS_FIXEDFILEINFO(vstruct.VStruct):
    def __init__(self):
        vstruct.VStruct.__init__(self)
        self.Signature          = v_uint32()
        self.StrucVersion       = v_uint32()
        self.FileVersionMS      = v_uint32()
        self.FileVersionLS      = v_uint32()
        self.ProductVersionMS   = v_uint32()
        self.ProductVersionLS   = v_uint32()
        self.FileFlagsMask      = v_uint32()
        self.FileFlags          = v_uint32()
        self.FileOS             = v_uint32()
        self.FileType           = v_uint32()
        self.FileSubtype        = v_uint32()
        self.FileDateMS         = v_uint32()
        self.FileDateLS         = v_uint32()

class v_name(v_bytes):
    '''
    Specifically for things that *should* be utf-8 strings, but aren't really enforced as such
    '''
    def __repr__(self):
        try:
            name = self._vs_value.decode('utf-8')
            return name
        except UnicodeDecodeError:
            return binascii.hexlify(self._vs_value).decode('utf-8')

    __str__ = __repr__

    def vsGetValue(self):
        '''
        Get the type specific value for this field.
        (Used by the structure dereference method to return
        a python native for the field by name)
        '''
        try:
            name = self._vs_value.decode('utf-8')
            return name
        except UnicodeDecodeError:
            return "[invalid name] %r" % self._vs_value

    def isValid(self):
        try:
            self._vs_value.decode('utf-8')
            return True
        except UnicodeDecodeError:
            return False

class IMAGE_SECTION_HEADER(vstruct.VStruct):

    def __init__(self):
        vstruct.VStruct.__init__(self)
        self.Name                 = v_name(8)
        self.VirtualSize          = v_uint32()
        self.VirtualAddress       = v_uint32()
        self.SizeOfRawData        = v_uint32()
        self.PointerToRawData     = v_uint32()
        self.PointerToRelocations = v_uint32()
        self.PointerToLineNumbers = v_uint32()
        self.NumberOfRelocations  = v_uint16()
        self.NumberOfLineNumbers  = v_uint16()
        self.Characteristics      = v_uint32()

    def isNameValid(self):
        # we've gotta bypass
        name = self.vsGetField('Name')
        return name.isValid()

class IMAGE_RUNTIME_FUNCTION_ENTRY(vstruct.VStruct):
    """
    Used in the .pdata section of a PE32+ for all non
    leaf functions.
    """
    def __init__(self):
        vstruct.VStruct.__init__(self)
        self.BeginAddress = v_uint32()
        self.EndAddress = v_uint32()
        self.UnwindInfoAddress = v_uint32()

class UNWIND_INFO(vstruct.VStruct):
    def __init__(self):
        vstruct.VStruct.__init__(self)
        self.VerFlags       = v_uint8()
        self.SizeOfProlog   = v_uint8()
        self.CountOfCodes   = v_uint8()
        self.FrameRegister  = v_uint8()

class SignatureEntry(vstruct.VStruct):

    def __init__(self):
        vstruct.VStruct.__init__(self)

        self.size = v_int32(bigend=False)

        # should always be 0x00020200
        self.magic = v_bytes(size=4)

        self.pkcs7 = v_bytes()

    def pcb_size(self):
        size = self.size
        self.vsGetField('pkcs7').vsSetLength(size)



# APIs for Windows 32-bit MSVC 2010 runtime library (msvcr100).
# Format:  rettype, retname, callconv, exactname, arglist(type, name)
#          arglist type is one of ['int', 'void *']
#          arglist name is one of [None, 'funcptr', 'obj', 'ptr']
#
# The subset of functions defined below accounts for 80% of observed calls.

api_defs = {
    'msvcr100.??2@yapaxi@z':( 'int', None, 'cdecl', 'msvcr100.??2@YAPAXI@Z', (('int', None),) ),
    'msvcr100.??_u@yapaxi@z':( 'int', None, 'cdecl', 'msvcr100.??_U@YAPAXI@Z', (('int', None),) ),
    'msvcr100.??3@yaxpax@z':( 'void', None, 'cdecl', 'msvcr100.??3@YAXPAX@Z', (('void *', 'ptr'),) ),
    'msvcr100.??_v@yaxpax@z':( 'void', None, 'cdecl', 'msvcr100.??_V@YAXPAX@Z', (('void *', 'ptr'),) ),

    'msvcr100._cxxthrowexception':( 'void', None, 'stdcall', 'msvcr100._CxxThrowException', (('void *', 'ptr'), ('int', None)) ),
    'msvcr100.__iob_func':( 'int', None, 'cdecl', 'msvcr100.__iob_func', () ),
    'msvcr100._invalid_parameter_noinfo':( 'void', None, 'cdecl', 'msvcr100._invalid_parameter_noinfo', () ),
    'msvcr100._errno':( 'int', None, 'cdecl', 'msvcr100._errno', () ),

    'msvcr100._exit':( 'int', None, 'cdecl', 'msvcr100._exit', (('int', None),) ),
    'msvcr100.abort':( 'int', None, 'cdecl', 'msvcr100.abort', () ),
    'msvcr100.exit':( 'int', None, 'cdecl', 'msvcr100.exit', (('int', None),) ),
    'msvcr100.quick_exit':( 'int', None, 'cdecl', 'msvcr100.quick_exit', (('int', None),) ),

    'msvcr100.calloc':( 'int', None, 'cdecl', 'msvcr100.calloc', (('int', None), ('int', None)) ),
    'msvcr100.malloc':( 'int', None, 'cdecl', 'msvcr100.malloc', (('int', None),) ),
    'msvcr100.free':( 'void', None, 'cdecl', 'msvcr100.free', (('void *', 'ptr'),) ),

    'msvcr100.memcmp':( 'int', None, 'cdecl', 'msvcr100.memcmp', (('void *', 'ptr'), ('void *', 'ptr'), ('int', None)) ),
    'msvcr100.memcpy':( 'int', None, 'cdecl', 'msvcr100.memcpy', (('void *', 'ptr'), ('void *', 'ptr'), ('int', None)) ),
    'msvcr100.memmove':( 'int', None, 'cdecl', 'msvcr100.memmove', (('void *', 'ptr'), ('void *', 'ptr'), ('int', None)) ),
    'msvcr100.memmove_s':( 'int', None, 'cdecl', 'msvcr100.memmove_s', (('void *', 'ptr'), ('int', None), ('void *', 'ptr'), ('int', None)) ),
    'msvcr100.memset':( 'int', None, 'cdecl', 'msvcr100.memset', (('void *', 'ptr'), ('int', None), ('int', None)) ),

    'msvcr100._snprintf':( 'int', None, 'cdecl', 'msvcr100._snprintf', (('int', None), ('int', None), ('int', None)) ),
    'msvcr100._swprintf':( 'int', None, 'cdecl', 'msvcr100._snprintf', (('int', None), ('int', None), ('int', None), ('int', None)) ),
    'msvcr100.fprintf':( 'int', None, 'cdecl', 'msvcr100.fprintf', (('void *', 'ptr'), ('int', None)) ),
    'msvcr100.sprintf':( 'int', None, 'cdecl', 'msvcr100.sprintf', (('int', None), ('int', None)) ),
    'msvcr100.printf':( 'int', None, 'cdecl', 'msvcr100.printf', (('int', None),) ),

    'msvcr100._stricmp':( 'int', None, 'cdecl', 'msvcr100._stricmp', (('void *', 'ptr'), ('void *', 'ptr')) ),
    'msvcr100.strchr':( 'int', None, 'cdecl', 'msvcr100.strchr', (('void *', 'ptr'), ('int', None)) ),
    'msvcr100.strcmp':( 'int', None, 'cdecl', 'msvcr100.strcmp', (('void *', 'ptr'), ('void *', 'ptr')) ),
    'msvcr100.strcpy':( 'int', None, 'cdecl', 'msvcr100.strcpy', (('void *', 'ptr'), ('void *', 'ptr')) ),
    'msvcr100.strlen':( 'int', None, 'cdecl', 'msvcr100.strlen', (('void *', 'ptr'),) ),
    'msvcr100.strncmp':( 'int', None, 'cdecl', 'msvcr100.strncmp', (('void *', 'ptr'), ('void *', 'ptr'), ('int', None)) ),
    'msvcr100.strncpy':( 'int', None, 'cdecl', 'msvcr100.strncpy', (('void *', 'ptr'), ('void *', 'ptr'), ('int', None)) ),
    'msvcr100.strstr':( 'int', None, 'cdecl', 'msvcr100.strstr', (('void *', 'ptr'), ('void *', 'ptr')) ),
    'msvcr100._wcsicmp':( 'int', None, 'cdecl', 'msvcr100._wcsicmp', (('void *', 'ptr'), ('void *', 'ptr')) ),
    'msvcr100.wcslen':( 'int', None, 'cdecl', 'msvcr100.wcslen', (('void *', 'ptr'),) ),
    'msvcr100.wcsrchr':( 'int', None, 'cdecl', 'msvcr100.wcsrchr', (('void *', 'ptr'), ('int', None)) ),

    'msvcr100._time64':( 'int', None, 'cdecl', 'msvcr100._time64', (('void *', 'ptr'),) ),
    'msvcr100.atoi':( 'int', None, 'cdecl', 'msvcr100.atoi', (('void *', 'ptr'),) ),
    'msvcr100.getenv':( 'int', None, 'cdecl', 'msvcr100.getenv', (('void *', 'ptr'),) ),
    'msvcr100.fclose':( 'int', None, 'cdecl', 'msvcr100.fclose', (('void *', 'ptr'),) ),
    'msvcr100.bsearch':( 'int', None, 'cdecl', 'msvcr100.bsearch', (('void *', 'ptr'), ('void *', 'ptr'), ('int', None), ('int', None), ('void *', 'funcptr')) )
    }



'''
Code for helping out windows kernel debugging...
'''
import logging

import vtrace
import vstruct
import vstruct.builder as vs_builder

from vstruct.primitives import *

logger = logging.getLogger(__name__)

class KeBugCheckBreak(vtrace.Breakpoint):

    def __init__(self, symname):
        vtrace.Breakpoint.__init__(self, None, expression=symname)
        self.stealthbreak = True # No NOTIFY_BREAK

    def notify(self, event, trace):
        sp = trace.getStackCounter()
        savedpc, exccode = trace.readMemoryFormat(sp, '<PP')
        trace._fireSignal(exccode)

win_builds = {
    2600: 'Windows XP',
}

class KDDEBUGGER_DATA64(vstruct.VStruct):

    def __init__(self):
        vstruct.VStruct.__init__(self)
        self._vs_field_align = True

        List = vstruct.VStruct()
        List.Flink   = v_ptr64()
        List.Blink   = v_ptr64()

        Header = vstruct.VStruct()
        Header.List     = List
        Header.OwnerTag = v_uint32()
        Header.Size     = v_uint32()

        self.Header = Header

        self.KernBase  = v_ptr64()
        self.BreakpointWithStatus  = v_ptr64()
        self.SavedContext  = v_ptr64()
        self.ThCallbackStack = v_uint16()
        self.NextCallback    = v_uint16()
        self.FramePointer    = v_uint16()
        #self.PaeEnabled:1    = v_uint16()
        self.PaeEnabled      = v_uint16()
        self.KiCallUserMode    = v_ptr64()
        self.KeUserCallbackDispatcher  = v_ptr64()
        self.PsLoadedModuleList    = v_ptr64()
        self.PsActiveProcessHead   = v_ptr64()
        self.PspCidTable   = v_ptr64()
        self.ExpSystemResourcesList    = v_ptr64()
        self.ExpPagedPoolDescriptor    = v_ptr64()
        self.ExpNumberOfPagedPools = v_ptr64()
        self.KeTimeIncrement   = v_ptr64()
        self.KeBugCheckCallbackListHead    = v_ptr64()
        self.KiBugcheckData    = v_ptr64()
        self.IopErrorLogListHead   = v_ptr64()
        self.ObpRootDirectoryObject    = v_ptr64()
        self.ObpTypeObjectType = v_ptr64()
        self.MmSystemCacheStart    = v_ptr64()
        self.MmSystemCacheEnd  = v_ptr64()
        self.MmSystemCacheWs   = v_ptr64()
        self.MmPfnDatabase = v_ptr64()
        self.MmSystemPtesStart = v_ptr64()
        self.MmSystemPtesEnd   = v_ptr64()
        self.MmSubsectionBase  = v_ptr64()
        self.MmNumberOfPagingFiles = v_ptr64()
        self.MmLowestPhysicalPage  = v_ptr64()
        self.MmHighestPhysicalPage = v_ptr64()
        self.MmNumberOfPhysicalPages   = v_ptr64()
        self.MmMaximumNonPagedPoolInBytes  = v_ptr64()
        self.MmNonPagedSystemStart = v_ptr64()
        self.MmNonPagedPoolStart   = v_ptr64()
        self.MmNonPagedPoolEnd = v_ptr64()
        self.MmPagedPoolStart  = v_ptr64()
        self.MmPagedPoolEnd    = v_ptr64()
        self.MmPagedPoolInformation    = v_ptr64()
        self.MmPageSize    = v_ptr64()
        self.MmSizeOfPagedPoolInBytes  = v_ptr64()
        self.MmTotalCommitLimit    = v_ptr64()
        self.MmTotalCommittedPages = v_ptr64()
        self.MmSharedCommit    = v_ptr64()
        self.MmDriverCommit    = v_ptr64()
        self.MmProcessCommit   = v_ptr64()
        self.MmPagedPoolCommit = v_ptr64()
        self.MmExtendedCommit  = v_ptr64()
        self.MmZeroedPageListHead  = v_ptr64()
        self.MmFreePageListHead    = v_ptr64()
        self.MmStandbyPageListHead = v_ptr64()
        self.MmModifiedPageListHead    = v_ptr64()
        self.MmModifiedNoWritePageListHead = v_ptr64()
        self.MmAvailablePages  = v_ptr64()
        self.MmResidentAvailablePages  = v_ptr64()
        self.PoolTrackTable    = v_ptr64()
        self.NonPagedPoolDescriptor    = v_ptr64()
        self.MmHighestUserAddress  = v_ptr64()
        self.MmSystemRangeStart    = v_ptr64()
        self.MmUserProbeAddress    = v_ptr64()
        self.KdPrintCircularBuffer = v_ptr64()
        self.KdPrintCircularBufferEnd  = v_ptr64()
        self.KdPrintWritePointer   = v_ptr64()
        self.KdPrintRolloverCount  = v_ptr64()
        self.MmLoadedUserImageList = v_ptr64()
        # NT 5.1 Addition
        self.NtBuildLab    = v_ptr64()
        self.KiNormalSystemCall    = v_ptr64()
        # NT 5.0 QFE addition
        self.KiProcessorBlock  = v_ptr64()
        self.MmUnloadedDrivers = v_ptr64()
        self.MmLastUnloadedDriver  = v_ptr64()
        self.MmTriageActionTaken   = v_ptr64()
        self.MmSpecialPoolTag  = v_ptr64()
        self.KernelVerifier    = v_ptr64()
        self.MmVerifierData    = v_ptr64()
        self.MmAllocatedNonPagedPool   = v_ptr64()
        self.MmPeakCommitment  = v_ptr64()
        self.MmTotalCommitLimitMaximum = v_ptr64()
        self.CmNtCSDVersion    = v_ptr64()
        # NT 5.1 Addition
        self.MmPhysicalMemoryBlock = v_ptr64()
        self.MmSessionBase = v_ptr64()
        self.MmSessionSize = v_ptr64()
        self.MmSystemParentTablePage   = v_ptr64()
        # Server 2003 addition
        self.MmVirtualTranslationBase  = v_ptr64()
        self.OffsetKThreadNextProcessor    = v_uint16()
        self.OffsetKThreadTeb  = v_uint16()
        self.OffsetKThreadKernelStack  = v_uint16()
        self.OffsetKThreadInitialStack = v_uint16()
        self.OffsetKThreadApcProcess   = v_uint16()
        self.OffsetKThreadState    = v_uint16()
        self.OffsetKThreadBStore   = v_uint16()
        self.OffsetKThreadBStoreLimit  = v_uint16()
        self.SizeEProcess  = v_uint16()
        self.OffsetEprocessPeb = v_uint16()
        self.OffsetEprocessParentCID   = v_uint16()
        self.OffsetEprocessDirectoryTableBase  = v_uint16()
        self.SizePrcb  = v_uint16()
        self.OffsetPrcbDpcRoutine  = v_uint16()
        self.OffsetPrcbCurrentThread   = v_uint16()
        self.OffsetPrcbMhz = v_uint16()
        self.OffsetPrcbCpuType = v_uint16()
        self.OffsetPrcbVendorString    = v_uint16()
        self.OffsetPrcbProcStateContext    = v_uint16()
        self.OffsetPrcbNumber  = v_uint16()
        self.SizeEThread   = v_uint16()
        self.KdPrintCircularBufferPtr  = v_ptr64()
        self.KdPrintBufferSize = v_ptr64()
        self.KeLoaderBlock = v_ptr64()
        self.SizePcr   = v_uint16()
        self.OffsetPcrSelfPcr  = v_uint16()
        self.OffsetPcrCurrentPrcb  = v_uint16()
        self.OffsetPcrContainedPrcb    = v_uint16()
        self.OffsetPcrInitialBStore    = v_uint16()
        self.OffsetPcrBStoreLimit  = v_uint16()
        self.OffsetPcrInitialStack = v_uint16()
        self.OffsetPcrStackLimit   = v_uint16()
        self.OffsetPrcbPcrPage = v_uint16()
        self.OffsetPrcbProcStateSpecialReg = v_uint16()
        self.GdtR0Code = v_uint16()
        self.GdtR0Data = v_uint16()
        self.GdtR0Pcr  = v_uint16()
        self.GdtR3Code = v_uint16()
        self.GdtR3Data = v_uint16()
        self.GdtR3Teb  = v_uint16()
        self.GdtLdt    = v_uint16()
        self.GdtTss    = v_uint16()
        self.Gdt64R3CmCode = v_uint16()
        self.Gdt64R3CmTeb  = v_uint16()
        self.IopNumTriageDumpDataBlocks    = v_ptr64()
        self.IopTriageDumpDataBlocks   = v_ptr64()
        # Longhorn
        self.VfCrashDataBlock  = v_ptr64()
        self.MmBadPagesDetected    = v_ptr64()
        self.MmZeroedPageSingleBitErrorsDetected   = v_ptr64()
        # Windows 7
        self.MmBadPagesDetected    = v_ptr64()
        self.OffsetPrcbContext = v_uint16()

def addBugCheckBreaks(trace):
    trace.addBreakpoint(KeBugCheckBreak('nt.KeBugCheck'))
    trace.addBreakpoint(KeBugCheckBreak('nt.KeBugCheckEx'))

def initWinkernTrace(trace, kpcrva):

    # FIXME snap in structs depending on version
    import vstruct.defs.windows.win_5_1_i386.ntoskrnl as vs_w_ntoskrnl
    trace.vsbuilder.addVStructNamespace('nt', vs_w_ntoskrnl) # FIXME no remote!

    b =vs_builder.VStructBuilder()
    b.addVStructCtor('KDDEBUGGER_DATA64',KDDEBUGGER_DATA64)
    trace.vsbuilder.addVStructNamespace('winkern',b)
    # Create a "struct" namespace "winkern" for our custom defined structures.

    trace.casesens = False

    kpcr = trace.getStruct('nt.KPCR', kpcrva)

    kver = trace.getStruct('nt.DBGKD_GET_VERSION64', kpcr.KdVersionBlock)
    dbgdata64va = trace.readMemoryFormat(kver.DebuggerDataList & trace.bigmask,'<I')[0]

    dbgdata64 = KDDEBUGGER_DATA64()
    dbgdata64.vsParse( trace.readMemory( dbgdata64va, len(dbgdata64) ))

    winver = win_builds.get( kver.MinorVersion )
    if winver is None:
        winver = 'Untested Windows Build! (%d)' % kver.MinorVersion

    kernbase = kver.KernBase & trace.bigmask
    modlist = kver.PsLoadedModuleList & trace.bigmask

    # FIXME hard coded page sizes!
    trace.setMeta('PAGE_SIZE', 1024)
    trace.setMeta('PAGE_SHIFT', 12)

    trace.setVariable('kpcr', kpcrva)
    trace.setVariable('kddebuggerdata64',dbgdata64va)
    trace.setVariable('KernelBase', kernbase)
    trace.setVariable('PsLoadedModuleList', modlist)

    trace.fireNotifiers(vtrace.NOTIFY_ATTACH)

    trace.addLibraryBase('nt', kernbase, always=True)

    ldr_entry = trace.readMemoryFormat(modlist, '<P')[0]
    while ldr_entry != modlist:
        ldte = trace.getStruct('nt.LDR_DATA_TABLE_ENTRY', ldr_entry)
        try:
            dllname = trace.readMemory(ldte.FullDllName.Buffer, ldte.FullDllName.Length).decode('utf-16le')
            dllbase = ldte.DllBase & trace.bigmask
            trace.addLibraryBase(dllname, dllbase, always=True)
        except Exception as e:
            logger.warning('Trouble while parsing one...')
        ldr_entry = ldte.InLoadOrderLinks.Flink & trace.bigmask

    addBugCheckBreaks(trace)

def walkListEntry(trace, va, sname, fname):
    '''
    Walk a list of structures (sname) with embedded
    LIST_ENTRY structs at field (fname) and yield each
    containing (va,structure).
    '''
    s = trace.getStruct(sname)
    listoff = s.vsGetOffset(fname)
    listva = va
    while True:
        structva = listva - listoff
        yield structva,trace.getStruct(sname,structva)
        nextva,prevva = trace.readMemoryFormat(listva,'<PP')
        if nextva == va:
            break

        listva = nextva

def walkListEntryHead(trace, va, sname, fname):
    '''
    Walk a list of structures (sname) with embedded
    LIST_ENTRY structs at field (fname) and yield each
    containing (va,structure).
    '''
    s = trace.getStruct(sname)
    listoff = s.vsGetOffset(fname)
    listva,prevva = trace.readMemoryFormat(va,'<PP')

    while True:
        structva = listva - listoff
        yield structva,trace.getStruct(sname,structva)
        nextva,prevva = trace.readMemoryFormat(listva,'<PP')
        if nextva == va:
            break

        listva = nextva




import os
import ctypes

def getHostId():
    sysdrive = os.getenv('SystemDrive') + '\\'
    buf = ctypes.create_string_buffer(64)
    if not ctypes.windll.kernel32.GetVolumeNameForVolumeMountPointA(sysdrive, buf, ctypes.c_uint32(64)):
        return None
    return buf.value




import unittest
import vivisect.tests.helpers as helpers

class HelperTest(unittest.TestCase):

    def test_helper_pathjoin(self):
        # retrieve a known vivtestfiles path ( or skip )
        helpers.getTestPath('windows','i386','helloworld.exe')




import logging
import unittest
import vivisect.tests.helpers as helpers
import vivisect.tests.utils as v_t_utils


logger = logging.getLogger(__name__)


class VivisectMultiFileTest(v_t_utils.VivTest):
    def test_multiple_i386(self):
        '''
        Note: because we're using a non-relocatable executable (vdir.llvm) and relocating it, this
        will unfortunately throw a lot of ugly exceptions, because there are hard-coded addresses
        in the instructions themselves, which are relocations because the file isn't "relocatable"

        We only test the things which should work because they are made available for relocation
        through the ELF metadata.
        '''
        vw = helpers.getTestWorkspace_nocache('linux', 'i386', 'chgrp.llvm')
        vw = helpers.getTestWorkspace_nocache('linux', 'i386', 'vdir.llvm', vw=vw)
        vw = helpers.getTestWorkspace_nocache('linux', 'i386', 'ld-2.31.so', vw=vw)

        # test memory maps are not overlapping
        for mmap1 in vw.getMemoryMaps():
            for mmap2 in vw.getMemoryMaps():
                if mmap1 == mmap2:
                    continue

                mmva1, mmsz1, mmp1, mmnm1 = mmap1
                mmva2, mmsz2, mmp2, mmnm2 = mmap2
                self.assertFalse(mmva1 <= mmva2 < (mmva1 + mmsz1))
                self.assertFalse(mmva2 <= mmva1 < (mmva2 + mmsz2))

        # test __entry values
        vdir = vw.parseExpression('vdir')
        chgrp = vw.parseExpression('chgrp')
        ld_2_31 = vw.parseExpression('ld_2_31')

        vdir_entry = vw.parseExpression('vdir.__entry')
        chgrp_entry = vw.parseExpression('chgrp.__entry')
        ld_2_31_entry = vw.parseExpression('ld_2_31.__entry')

        self.assertEqual(vdir_entry-vdir, 7024)
        self.assertEqual(chgrp_entry-chgrp, 4576)
        self.assertEqual(ld_2_31_entry-ld_2_31, 4384)

        # test main functions
        vdir_main = vw.parseExpression('vdir.main')
        chgrp_main = vw.parseExpression('chgrp.main')

        self.assertEqual(vdir_main-vdir, 7312)
        self.assertEqual(chgrp_main-chgrp, 5712)

        # test PLT functions
        vdir_plt_names = [(va,name) for va,name in vw.getNames() if 'vdir.plt_' in name]
        chgrp_plt_names = [(va,name) for va,name in vw.getNames() if 'chgrp.plt_' in name]
        ld_2_31_plt_names = [(va,name) for va,name in vw.getNames() if 'ld_2_31.plt_' in name]
        self.assertEqual(len(vdir_plt_names), 0)    # can't expect PLT analysis to work for a binary not intended to be relocated.  
        self.assertEqual(len(chgrp_plt_names), 70)  # this is more than we get when loading chgrp on it's own!?
        self.assertEqual(len(ld_2_31_plt_names), 8)

        ## now just poke at one or two in each proggy
        ### chgrp __gmon_start__
        got_gmon_start = vw.parseExpression('chgrp+0xbffc')
        plt_gmon_start = vw.parseExpression('chgrp+0x11d0')
        self.assertIn('*.__gmon_start___', vw.getName(got_gmon_start))
        self.assertIn('plt___gmon_start__', vw.getName(plt_gmon_start))
        self.assertIn((plt_gmon_start, got_gmon_start, 2, 0), vw.getXrefsFrom(plt_gmon_start))

        ### chgrp __assert_fail
        got_gmon_start = vw.parseExpression('chgrp+0xbffc')
        plt_gmon_start = vw.parseExpression('chgrp+0x11d0')
        self.assertIn('*.__gmon_start___', vw.getName(got_gmon_start))
        self.assertIn('plt___gmon_start__', vw.getName(plt_gmon_start))
        self.assertIn((plt_gmon_start, got_gmon_start, 2, 0), vw.getXrefsFrom(plt_gmon_start))

        # test exports
        for exp in vw.getExports():
            eva, etype, esym, efile = exp
            mmap = vw.getMemoryMap(eva)
            self.assertEqual(mmap[3], efile, msg="Failed: %r  for %r" % (exp, mmap))

        # test GOT locations
        vdir_main = vw.parseExpression('vdir.main')
        chgrp_main = vw.parseExpression('chgrp.main')

        self.assertEqual(vdir_main-vdir, 7312)
        self.assertEqual(chgrp_main-chgrp, 5712)

        # test Relocs
        self.assertIn(('ld_2_31', 179948, 2, 135749, 4), vw.getRelocations())
        rel0 = vw.parseExpression('ld_2_31 + 179948')
        ptr0 = vw.parseExpression('ld_2_31 + 135749')
        self.assertIn((rel0, ptr0, 3, 0), vw.getXrefsFrom(rel0))


        # test Sections
        for seg in vw.getSegments():
            if seg[2] == '.text':
                if seg[3] == 'vdir':
                    self.assertEqual(7024, seg[0] - vw.parseExpression('vdir'))
                if seg[3] == 'chgrp':
                    self.assertEqual(4576, seg[0] - vw.parseExpression('chgrp'))
                if seg[3] == 'ld_2_31':
                    self.assertEqual(4352, seg[0] - vw.parseExpression('ld_2_31'))



'''
Home of some helpers for python interactive stuff.
'''
import types
import traceback

from threading import Thread
from PyQt5.QtWidgets import *

from vqt.main import idlethread
from vqt.basics import *
from vqt.common import scripterr


class ScriptThread(Thread):

    def __init__(self, cobj, locals):
        Thread.__init__(self, daemon=True)
        self.cobj = cobj
        self.locals = locals

    def run(self):
        try:
            exec(self.cobj, self.locals)
        except Exception as e:
            scripterr(str(e), traceback.format_exc())


class VQPythonView(QWidget):

    def __init__(self, locals=None, parent=None):
        if locals is None:
            locals = {}

        self._locals = locals

        QWidget.__init__(self, parent=parent)

        self._textWidget = QTextEdit(parent=self)
        self._botWidget = QWidget(parent=self)
        self._help_button = QPushButton('?', parent=self._botWidget)
        self._run_button = QPushButton('Run', parent=self._botWidget)
        self._run_button.clicked.connect(self._okClicked)
        self._help_button.clicked.connect(self._helpClicked)

        self._help_text = None

        hbox = HBox(None, self._help_button, self._run_button)
        self._botWidget.setLayout(hbox)

        vbox = VBox(self._textWidget, self._botWidget)
        self.setLayout(vbox)

        self.setWindowTitle('Python Interactive')

    def _okClicked(self):
        pycode = str(self._textWidget.document().toPlainText())
        try:
            cobj = compile(pycode, "vqpython_exec.py", "exec")
            sthr = ScriptThread(cobj, self._locals)
            sthr.start()
        except:
            exceptstr = traceback.format_exc()
            scripterr("Can't Compile", exceptstr)

    def _helpClicked(self):
        withhelp = []
        for lname, lval in self._locals.items():
            if type(lval) in (types.ModuleType, ):
                continue
            doc = getattr(lval, '__doc__', '\nNo Documentation\n')
            if doc is None:
                doc = '\nNo Documentation\n'
            withhelp.append((lname, doc))

        withhelp.sort()

        txt = 'Objects/Functions in the namespace:\n'
        for name, doc in withhelp:
            txt += ('====== %s\n' % name)
            txt += ('%s\n' % doc)

        self._help_text = QTextEdit()
        self._help_text.setReadOnly(True)
        self._help_text.setWindowTitle('Python Interactive Help')
        self._help_text.setText(txt)
        self._help_text.show()
